from __future__ import unicode_literals

from django_tables2 import SingleTableMixin

from evoteajaxforms.utils import add_errors_prefix_formset
from guardian import mixins


class DjangoTablesFilterMixin(SingleTableMixin):
    """
    Allows to add filters to the table
    """
    filter_class = None

    def get_filter(self, qs):
        return self.filter_class(self.request.GET, qs)

    def get_context_data(self, **kwargs):
        context = super(DjangoTablesFilterMixin, self)\
            .get_context_data(**kwargs)
        table_filter = self.get_filter(self.get_queryset())
        context.update({
            'filter': table_filter
        })
        return context

    def get_table_data(self):
        """ """
        table_filter = self.get_filter(self.get_queryset())
        return table_filter.qs


class ModelFieldChangedMixin(object):
    """ """
    _changed_prefix = "_prev_"

    def track_changes(self, fields):
        """ """
        for field in fields:
            if hasattr(self, field):
                setattr(self, self.get_track_field_name(field),
                        getattr(self, field))

    def has_changed(self, field):
        if hasattr(self, field):
            value = getattr(self, field)
            prev_value = getattr(self, self.get_track_field_name(field))
            return value != prev_value
        return False

    def get_prev_field(self, field):
        if hasattr(self, self.get_track_field_name(field)):
            return getattr(self, self.get_track_field_name(field))
        return None

    def get_track_field_name(self, field):
        return "{}{}".format(self._changed_prefix, field)


class NestedFormsetFormMixin(object):

    formset_class = None
    formset_prefix = None

    def __init__(self, *args, **kwargs):
        # data and files need to be declared here because if equals to None,
        # the init will change it to dict and will mark form as bounded
        data = kwargs.get('data')
        files = kwargs.get('files')
        prefix = kwargs.get('prefix', self.prefix)
        # instance = kwargs.get('instance')
        super(NestedFormsetFormMixin, self).__init__(*args, **kwargs)
        # bind documents formset to the form instance
        self.formset = self.formset_class(
            data=data, files=files, queryset=self.get_queryset(),
            prefix=prefix + "-" + self.formset_prefix)

    def full_clean(self):
        """Inject errors of nested formset"""
        super(NestedFormsetFormMixin, self).full_clean()
        if not self.formset.is_valid():
            self._errors[self.formset_prefix] = add_errors_prefix_formset(
                self.formset.errors, self.formset.prefix)

    def has_changed(self):
        """Has changed reflects the formset too"""
        return super(NestedFormsetFormMixin, self).has_changed() or \
            self.formset.has_changed()


class PermissionListMixin(mixins.PermissionListMixin):
    """ """
    # HACK: login_url avoid errors on tests. never used, we always raise 403
    login_url = "/"
    get_objects_for_user_extra_kwargs = {
        "with_superuser": False,  # even superuser cant view all
        "use_groups": False  # avoid search on groups
    }


class PermissionRequiredMixin(mixins.PermissionRequiredMixin):
    """ """
    # HACK: this avoid errors on tests. never used, we always raise 403
    login_url = "/"
