from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django import forms
from django.db.models import Q
from django.conf import settings

from evotecore import widgets
from evoteusers.models import User

from .models import Message

from captcha.fields import ReCaptchaField

from .templatetags.messages_tags import nested_url
from .registers import recipient_register
from .utils import get_choices


class NewMessageBaseForm(forms.ModelForm):
    """ """
    name = forms.CharField()
    subject = forms.CharField(max_length=260)

    class Meta:
        model = Message
        fields = ('name', 'subject', 'body',)

    def __init__(self, *args, **kwargs):
        super(NewMessageBaseForm, self).__init__(*args, **kwargs)
        self.fields['name'].widget.attrs['placeholder'] = _('Name')
        self.fields['subject'].widget.attrs['placeholder'] = _('Subject')
        self.fields['body'].widget.attrs['placeholder'] = _('Message')
        self.fields['name'].required = True


class MessageAnonymousForm(NewMessageBaseForm):
    """ """
    captcha = ReCaptchaField()

    class Meta(NewMessageBaseForm.Meta):
        fields = NewMessageBaseForm.Meta.fields + \
            ('anonymous_email', 'captcha',)

    def __init__(self, *args, **kwargs):
        super(MessageAnonymousForm, self).__init__(*args, **kwargs)
        self.fields['anonymous_email'].widget.attrs['placeholder'] = _('Email')
        self.fields['anonymous_email'].required = True


class MessageAuthenticatedBaseForm(NewMessageBaseForm):
    """ """
    email = forms.EmailField()

    class Meta(NewMessageBaseForm.Meta):
        fields = NewMessageBaseForm.Meta.fields + ('email',)

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user')
        self.obj = kwargs.pop('obj', None)
        super(MessageAuthenticatedBaseForm, self).__init__(*args, **kwargs)
        for field in self.initial:
            curr_field = self.fields[field]
            if isinstance(curr_field.widget, widgets.MultipleSelect):
                curr_field.disabled = True
            else:
                curr_field.widget.attrs['readonly'] = True
        self.fields['email'].required = True

    def save(self, commit=True):
        instance = super(MessageAuthenticatedBaseForm, self).save(commit=False)
        email = self.cleaned_data['email']
        noreply = getattr(settings, "EVOTE_MAILING_NOREPLY_DEFAULT", '')
        if not email == noreply:
            user = User.objects.get(Q(email=email) | Q(notice_email=email))
        else:
            user = self.user
        instance.sender = user
        if commit:
            instance.save()
            self.save_m2m()
        return instance


class MessageAuthenticatedForm(MessageAuthenticatedBaseForm):
    """ """
    recipients = forms.MultipleChoiceField(widget=widgets.MultipleAjaxSelect())

    class Meta(MessageAuthenticatedBaseForm.Meta):
        fields = MessageAuthenticatedBaseForm.Meta.fields + ('recipients',)

    def __init__(self, *args, **kwargs):
        super(MessageAuthenticatedForm, self).__init__(*args, **kwargs)
        if self.obj:
            url = nested_url('message-recipients', slug=self.obj.slug)
        else:
            url = nested_url('message-recipients')
        self.fields['recipients'].widget.attrs.update({
            'data-select-ajax-url': url
        })
        if 'data' in kwargs:
            self.fields['recipients'].choices = \
                get_choices(recipient_register.get_all_recipients(
                    self.user, process=self.obj))
