from __future__ import unicode_literals

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible
from django_extensions.db.models import TimeStampedModel
from django.utils import timezone
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from autoslug import AutoSlugField

from .managers import MessageManager


@python_2_unicode_compatible
class Message(TimeStampedModel):

    class Folder:
        RECIEVED = 0
        SENT = 1
        ARCHIVED = 2
        TRASH = 3

        choices = (
            (RECIEVED, _("Recieved")),
            (SENT, _("Sent")),
            (ARCHIVED, _("Archived")),
            (TRASH, _("Trash")),
        )

    class State:
        NEW = 0
        READ = 1
        REPLIED = 2

        choices = (
            (NEW, _("New")),
            (READ, _("Read")),
            (REPLIED, _("Replied")),
        )

    subject = models.CharField(_("Subject"), max_length=260)
    body = models.TextField(_("Body"))
    read_at = models.DateTimeField(_("Read at"), null=True, blank=True)
    sender = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='sent',
        verbose_name=_("Sender"), null=True, blank=True)
    anonymous_email = models.EmailField(
        verbose_name=_("Sender"), null=True, blank=True)
    recipients = models.ManyToManyField(
        settings.AUTH_USER_MODEL, verbose_name=_("Recipients"))
    parent = models.ForeignKey(
        'self', related_name='siblings', null=True, blank=True,
        verbose_name=_("Parent message"))
    folder = models.PositiveSmallIntegerField(
        _("Folder"), choices=Folder.choices, default=Folder.SENT)
    state = models.PositiveSmallIntegerField(
        _("State"), choices=State.choices, default=State.NEW)
    slug = AutoSlugField(
        populate_from='subject', unique=True, editable=False)
    content_type = models.ForeignKey(ContentType, null=True)
    object_id = models.PositiveIntegerField(null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    objects = MessageManager()

    class Meta:
        ordering = ['-created']
        verbose_name = _("message")
        verbose_name_plural = _("messages")
        permissions = (
            ('view_message', 'View message'),
        )

    def __str__(self):
        return "{}".format(self.subject)

    @property
    def is_read(self):
        return bool(self.read_at)

    @property
    def is_archived(self):
        return self.folder == self.Folder.ARCHIVED

    def mark_as_read(self):
        if not self.read_at:
            self.read_at = timezone.now()
            self.save()
