from __future__ import unicode_literals

import json

from django.test import TestCase
from guardian.shortcuts import assign_perm

from evoteusers.tests.utils import UserTestUtils
from evotemessages.models import Message
from evotemessages.templatetags.messages_tags import nested_url
from evotemessages.registers import recipient_register
from evotevotings.processes.tests.utils import ProcessesTestUtils

from .utils import MessagesUtils


class MessagesRecievedViewTest(UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.url = nested_url('messages-recieved', slug='')

    def test_used_template(self):
        """ """
        self.login()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/evotemessages/message_list.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessagesSentViewTest(UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.url = nested_url('messages-sent', slug='')

    def test_used_template(self):
        """ """
        self.login()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/evotemessages/message_list.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessagesArchivedViewTest(UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.url = nested_url('messages-archived', slug='')

    def test_used_template(self):
        """ """
        self.login()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/evotemessages/message_list.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ArchiveMessageViewTest(MessagesUtils, UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.user = self.create_user(**self.credentials)
        self.message = self.create_message(
            sender=self.user, folder=Message.Folder.SENT)
        assign_perm("view_message", self.user, self.message)
        self.url = nested_url('archive-message', slug='', pk=self.message.pk)

    def test_archive_message(self):
        self.client.login(**self.credentials)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

        was_archived = Message.objects.filter(
            pk=self.message.pk, folder=Message.Folder.ARCHIVED).exists()
        self.assertEqual(was_archived, True)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class DeleteMessageViewTest(MessagesUtils, UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.user = self.create_user(**self.credentials)
        self.message = self.create_message(sender=self.user)
        assign_perm("view_message", self.user, self.message)
        self.url = nested_url('delete-message', slug='', pk=self.message.pk)

    def test_delete_message(self):
        """ """
        self.client.login(**self.credentials)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

        was_deleted = Message.objects.filter(
            pk=self.message.pk, folder=Message.Folder.TRASH).exists()
        self.assertEqual(was_deleted, True)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessageNewViewTest(ProcessesTestUtils, UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.credentials_sender = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.sender = self.create_user(**self.credentials_sender)
        self.process = self.create_electoral_process()
        self.member = self.create_electoral_member(
            electoral_process=self.process)
        self.url = nested_url('message-new', slug='')

    def test_used_template(self):
        """ """
        self.client.login(**self.credentials_sender)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/evotemessages/message_form.html")

    def test_form_valid_message_sent(self):
        """
        Test post of valid sent message
        and checks if it was delivered to all recipients
        and if is in the sentbox of user
        """
        setattr(self.sender, 'is_election_voter', True)
        self.client.login(**self.credentials_sender)
        session = self.client.session
        session['voter_user_profile'] = True
        session.save()
        self.recipient = recipient_register.get_all_recipients(
            self.sender, q=self.member.user.name, slug=[self.process.slug])[0]\
            .get('id')
        data = {
            'newmsg-name': self.sender.name,
            'newmsg-email': self.sender.email,
            'newmsg-recipients': [self.recipient],
            'newmsg-subject': 'Subject of message',
            'newmsg-body': 'body of message',
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

        in_sentbox = Message.objects.filter(
            subject=data['newmsg-subject'],
            folder=Message.Folder.SENT).exists()
        self.assertEqual(in_sentbox, True)

        all_recieved = True
        for destination in data['newmsg-recipients']:
            users = recipient_register.get_all_users(destination)
            if not Message.objects.filter(
                    recipients__in=users,
                    folder=Message.Folder.RECIEVED).exists():
                all_recieved = False
                break
        self.assertEqual(all_recieved, True)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessageViewTest(MessagesUtils, UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.user = self.create_user(**self.credentials)
        self.message = self.create_message()
        assign_perm("view_message", self.user, self.message)
        self.url = nested_url('message-view', slug='',
                              msg_slug=self.message.slug)

    def test_used_template(self):
        """ """
        self.client.login(**self.credentials)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/evotemessages/message_view.html")

    def test_access_not_permitted(self):
        """
        Tests if user can access to other user message
        """
        credentials = {
            "email": "dev2@eurotux.com",
            "password": "password"
        }
        self.create_user(**credentials)
        self.client.login(**credentials)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessageResponseViewTest(
        ProcessesTestUtils, MessagesUtils, UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.user = self.create_user(**self.credentials)
        setattr(self.user, 'is_election_voter', True)
        self.message = self.create_message()
        assign_perm("view_message", self.user, self.message)
        self.process = self.create_electoral_process()
        self.member = self.create_electoral_member(
            electoral_process=self.process)
        self.recipient = recipient_register.get_all_recipients(
            self.user, q=self.member.user.name, slug=[self.process.slug])[0]\
            .get('id')
        self.url = nested_url('message-response', slug='',
                              msg_slug=self.message.slug)

    def test_used_template(self):
        """ """
        self.client.login(**self.credentials)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/evotemessages/message_form.html")

    def test_form_valid_messages_sent(self):
        """
        Test post of valid response message
        and checks if it was delivered to all recipients
        and if is in the sentbox of user
        """
        data = {
            'remsg-name': self.user.name,
            'remsg-email': self.user.email,
            'remsg-subject': 'Re: ' + self.message.subject,
            'remsg-body': 'body of message',
        }
        self.client.login(**self.credentials)
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

        in_sentbox = Message.objects.filter(
            subject=data['remsg-subject'],
            folder=Message.Folder.SENT).exists()
        self.assertEqual(in_sentbox, True)

        recieved = Message.objects.filter(
            subject=data['remsg-subject'],
            recipients__in=[self.message.sender],
            folder=Message.Folder.RECIEVED).exists()
        self.assertEqual(recieved, True)

    def test_access_not_permitted(self):
        """
        Tests if user can access to other user message
        """
        credentials = {
            "email": "dev2@eurotux.com",
            "password": "password"
        }
        self.create_user(**credentials)
        self.client.login(**credentials)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)
