from __future__ import unicode_literals

from django.views.generic import ListView, DetailView
from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import login_required
from django.views.generic.base import View
from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import ugettext_lazy as _
from django.contrib import messages
from guardian.shortcuts import assign_perm, remove_perm

from evoteajaxforms.views import FormJsonSimpleView
from evotecore.mixins import PermissionRequiredMixin, PermissionListMixin

from .templatetags.messages_tags import nested_url
from .models import Message
from .mixins import MessageViewMixin, MessageBaseMixin
from .registers import recipient_register


class MessagesListBaseView(MessageBaseMixin, ListView):
    """ """
    model = Message
    paginate_by = 10
    folder = Message.Folder.TRASH
    template = "evotemessages/message_list.html"

    def get_context_data(self, *args, **kwargs):
        context = super(MessagesListBaseView, self).\
            get_context_data(*args, **kwargs)
        context.update({
            'is_inbox': self.folder == Message.Folder.RECIEVED,
            'is_sentbox': self.folder == Message.Folder.SENT,
            'is_archivebox': self.folder == Message.Folder.ARCHIVED,
        })
        return context


@method_decorator(login_required, name="dispatch")
class MessagesRecievedView(PermissionListMixin, MessagesListBaseView):
    """ """
    folder = Message.Folder.RECIEVED
    permission_required = "evotemessages.view_message"

    def get_queryset(self):
        return Message.objects.inbox(self.request.user, None)


@method_decorator(login_required, name="dispatch")
class MessagesSentView(PermissionListMixin, MessagesListBaseView):
    """ """
    folder = Message.Folder.SENT
    permission_required = "evotemessages.view_message"

    def get_queryset(self):
        return Message.objects.sentbox(self.request.user, None)


@method_decorator(login_required, name="dispatch")
class MessagesArchivedView(PermissionListMixin, MessagesListBaseView):
    """ """
    folder = Message.Folder.ARCHIVED
    permission_required = "evotemessages.view_message"

    def get_queryset(self):
        return Message.objects.archivebox(self.request.user, None)


@method_decorator(login_required, name="dispatch")
class ArchiveMessageView(PermissionRequiredMixin, View):
    permission_required = "evotemessages.view_message"

    def get_permission_object(self):
        msg_pk = self.kwargs.get('pk', None)
        return get_object_or_404(Message, pk=msg_pk)

    def get(self, request, *args, **kwargs):
        slug = kwargs.get('slug', '')
        message = self.get_permission_object()
        message.folder = Message.Folder.ARCHIVED
        message.save()
        messages.success(request, _("Message was archived"))
        url = nested_url('messages-recieved', slug=slug)
        return redirect(url)


@method_decorator(login_required, name="dispatch")
class DeleteMessageView(PermissionRequiredMixin, View):
    permission_required = "evotemessages.view_message"

    def get_permission_object(self):
        msg_pk = self.kwargs.get('pk', None)
        return get_object_or_404(Message, pk=msg_pk)

    def get(self, request, *args, **kwargs):
        slug = kwargs.get('slug', '')
        message = self.get_permission_object()
        message.folder = Message.Folder.TRASH
        message.save()
        remove_perm("view_message", self.request.user, message)
        messages.success(request, _("Message was deleted"))
        url = nested_url('messages-recieved', slug=slug)
        return redirect(url)


@method_decorator(login_required, name="dispatch")
class MessageNewView(MessageViewMixin, FormJsonSimpleView):
    """ """
    prefix = 'newmsg'
    template = "evotemessages/message_form.html"

    def get_success_url(self):
        return nested_url('messages-recieved')

    def form_valid(self, form):
        instance = form.save(commit=False)
        form_recipients = form.cleaned_data.get('recipients', [])
        recipients = []
        for name in form_recipients:
            recipients += recipient_register.get_all_users(name)
        instance.save()
        instance.recipients = recipients
        if instance.sender:
            assign_perm("view_message", instance.sender, instance)
        Message.objects.send(instance)
        messages.success(
            self.request, _("Your message was sent successfully."))
        return self.json_to_response()


@method_decorator(login_required, name="dispatch")
class MessageView(PermissionRequiredMixin, MessageBaseMixin, DetailView):
    """ """
    model = Message
    template = "evotemessages/message_view.html"
    slug_url_kwarg = "msg_slug"
    permission_required = "evotemessages.view_message"

    def get(self, request, *args, **kwargs):
        response = super(MessageView, self).get(request, *args, **kwargs)
        self.object.mark_as_read()
        return response


@method_decorator(login_required, name="dispatch")
class MessageResponseView(PermissionRequiredMixin, MessageViewMixin,
                          FormJsonSimpleView):
    """ """
    prefix = 'remsg'
    response = True
    template = "evotemessages/message_form.html"
    slug_url_kwarg = "msg_slug"
    permission_required = "evotemessages.view_message"

    def get_success_url(self):
        return nested_url('messages-recieved')

    def get_permission_object(self):
        return Message.objects.get(slug=self.kwargs[self.slug_url_kwarg])

    def form_valid(self, form):
        instance = form.save(commit=False)
        if 'msg_slug' in self.kwargs:
            parent = self.get_permission_object()
            instance.parent = parent
        instance.save()
        instance.recipients = [parent.sender]
        if instance.sender:
            assign_perm("view_message", instance.sender, instance)
        Message.objects.send(instance)
        messages.success(
            self.request, _("Your message was sent successfully."))
        return self.json_to_response()
