from __future__ import unicode_literals

from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import PermissionsMixin
from django.db.models.signals import post_init, post_save
from django.utils.translation import ugettext_lazy as _
from django.db import models

from django_extensions.db.models import TimeStampedModel
from phonenumber_field.modelfields import PhoneNumberField
from evotecore.utils import images_upload_to
from evotecore.mixins import ModelFieldChangedMixin

from .managers import UserManager
from .mailings import EmailChangeOldMail, MethodChangeMail

from sorl.thumbnail import ImageField


class User(PermissionsMixin, ModelFieldChangedMixin, AbstractBaseUser,
           TimeStampedModel):
    """Basic django user using email as username and without default
    django permissions
    """

    class NoticeMethod:
        SMS = 0
        EMAIL = 1

        choices = (
            (SMS, _("SMS")),
            (EMAIL, _("Email")),
        )

    email = models.EmailField(_("email address"), unique=True, db_index=True)
    name = models.CharField(_("name"), max_length=255, db_index=True)
    # is_staff and is_active added to admin compat propose
    is_staff = models.BooleanField(_("staff status"), default=False)
    is_active = models.BooleanField(_("active"), default=True)
    # specific evote profile fields
    is_institutional_responsible = models.BooleanField(
        _("institutional responsible status"), default=False)
    photo = ImageField(_("photo"), upload_to=images_upload_to, blank=True)
    primary_phone = PhoneNumberField(_("primary phone number"), blank=True)
    notice_method = models.PositiveSmallIntegerField(
        _("notice method"), choices=NoticeMethod.choices,
        default=NoticeMethod.EMAIL)
    # number used to notified user, defaults to primary if blank
    notice_phone = PhoneNumberField(
        _("notice phone number"), blank=True,
        help_text=_("Phone number user to send notifications. If not "
                    "defined, primary phone used instead"))
    # email used to notified user, defaults to user email if blank
    notice_email = models.EmailField(
        _("notice email"), blank=True,
        help_text=_("Email used to send notifications. If not defined, "
                    "primary email used instead"))

    objects = UserManager()

    USERNAME_FIELD = "email"

    def get_phone(self):
        """Use this property to obtain phone notice number instead of direct
        attribute notice_phone.
        """
        return self.notice_phone or self.primary_phone

    def get_email(self):
        """Use this property to obtain email notice instead of direct
        attribute email_phone.
        """
        return self.notice_email or self.email

    def get_full_name(self):
        return self.name

    def get_short_name(self):
        return self.name

    @staticmethod
    def post_init(sender, instance, **kwargs):
        """Callback staticmethod for post_init signal"""
        instance.track_changes([
            'notice_email', 'notice_method'
        ])

    @staticmethod
    def post_save(sender, instance, **kwargs):
        """Callback staticmethod for post_save signal"""
        if instance.id:
            if instance.has_changed('notice_email'):
                # Send email to old address
                old = instance.get_prev_field('notice_email')
                context = {
                    'old': old,
                    'new': instance.notice_email
                }
                EmailChangeOldMail(context).send([(instance.name, old)])

            if instance.has_changed('notice_method'):
                old_value = instance.get_prev_field('notice_method')
                old = User.NoticeMethod.choices[old_value][1]
                context = {
                    'old': old,
                    'new': instance.get_notice_method_display()
                }
                MethodChangeMail(context).send(
                    [(instance.name, instance.notice_email)])


post_init.connect(User.post_init, sender=User)
post_save.connect(User.post_save, sender=User)
