from __future__ import unicode_literals

from django.contrib.auth.decorators import login_required
from django.views.decorators.debug import sensitive_post_parameters
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect
from django.utils.translation import ugettext_lazy as _
from django.utils.decorators import method_decorator
from django.views.generic import FormView, RedirectView
from django.contrib.auth import REDIRECT_FIELD_NAME, login, logout
from django.utils.http import is_safe_url
from django.shortcuts import redirect
from django.contrib import messages
from django.conf import settings

from evoteajaxforms.views import FormJsonView, AjaxResponseAction
from evoteusers.forms import EvoteAuthenticationForm

from .models import User
from .forms import PersonalDataForm


class EvoteLoginView(FormView):
    """Provides user the ability to login
    """
    form_class = EvoteAuthenticationForm
    redirect_field_name = REDIRECT_FIELD_NAME

    @method_decorator(sensitive_post_parameters('password'))
    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated():
            return redirect(settings.LOGIN_REDIRECT_URL)
        # Sets a test cookie to make sure the user has cookies enabled
        request.session.set_test_cookie()
        return super(EvoteLoginView, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        login(self.request, form.get_user())
        # If the test cookie worked, go ahead and
        # delete it since its no longer needed
        if self.request.session.test_cookie_worked():
            self.request.session.delete_test_cookie()
        return super(EvoteLoginView, self).form_valid(form)

    def get_success_url(self):
        redirect_to = self.request.GET.get(self.redirect_field_name)
        if not is_safe_url(url=redirect_to, host=self.request.get_host()):
            redirect_to = settings.LOGIN_REDIRECT_URL
        return redirect_to


class EvoteLogoutView(RedirectView):
    """Provides users the ability to logout"""

    def get(self, request, *args, **kwargs):
        logout(request)
        return super(EvoteLogoutView, self).get(request, *args, **kwargs)

    def get_redirect_url(self):
        return settings.LOGIN_URL


@method_decorator(login_required, name="dispatch")
class PersonalDataBaseView(FormJsonView):
    """
    Shows the personal data view
    """
    model = User
    form_class = PersonalDataForm
    action = AjaxResponseAction.REFRESH
    prefix = 'personal'

    def get_object(self):
        return self.request.user

    def form_valid(self, form):
        messages.success(
            self.request, _("Data updated with success."))
        return super(PersonalDataBaseView, self).form_valid(form)
