from __future__ import unicode_literals

from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.conf import settings

from evotevotings.admin.models import Page
from evotevotings.admin.forms import AdminAuthenticationForm

from .utils import PageTestUtils
from . import AdminTestCase


class LoginViewTest(AdminTestCase):
    """ """

    def setUp(self):
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.user = self.create_user(is_staff=True, **self.credentials)

    def test_login_view(self):
        """ """
        response = self.client.get(reverse('evoteadmin:login'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'evoteadmin/login.html')

    def test_login_staff_user(self):
        """ """
        response = self.client.post(reverse('evoteadmin:login'), follow=True,
                                    data=self.credentials)
        self.assertRedirects(response, reverse('evoteadmin:index'))
        self.assertEqual(response.context[0]['user'], self.user)

    def test_login_non_staff_user(self):
        """ """
        self.user.is_staff = False
        self.user.save()
        response = self.client.post(reverse('evoteadmin:login'), follow=True,
                                    data=self.credentials)
        error_message = AdminAuthenticationForm.error_messages['invalid_login']
        self.assertFormError(response, "form", None, error_message)

    def test_login_with_wrong_credentials(self):
        self.credentials['password'] = "wrong_password"
        response = self.client.post(reverse('evoteadmin:login'),
                                    data=self.credentials)
        error_message = AdminAuthenticationForm.error_messages['invalid_login']
        self.assertFormError(response, "form", None, error_message)


class LogoutViewTest(AdminTestCase):

    def setUp(self):
        # prepare a logged in voter
        self.login_as_admin()

    def test_logout_view(self):
        """ """
        response = self.client.get(reverse('evoteadmin:logout'), follow=True)
        self.assertRedirects(response, settings.LOGIN_URL)
        self.assertIsInstance(response.context[0]['user'], AnonymousUser)


class HomepageViewTest(AdminTestCase):

    def setUp(self):
        self.login_as_admin()
        self.url = reverse("evoteadmin:index")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "evoteadmin/index.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class PageListViewTest(AdminTestCase):

    def setUp(self):
        self.login_as_admin()
        self.url = reverse("evoteadmin:page-list")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response,
            "evoteadmin/page_list.html"
        )

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class PageDetailViewTest(PageTestUtils, AdminTestCase):

    def setUp(self):
        self.login_as_admin()
        instance = self.create_page(
            created_by=self.user
        )
        instance.save()
        self.url = reverse(
            "evoteadmin:page-update",
            kwargs={'slug': instance.slug}
        )

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response,
            "evoteadmin/page_update.html"
        )

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class PageDeleteViewTest(PageTestUtils, AdminTestCase):

    def setUp(self):
        self.login_as_admin()
        instance = self.create_page(created_by=self.user)
        self.instance_slug = instance.slug
        self.url = reverse(
            "evoteadmin:page-delete",
            kwargs={'slug': instance.slug}
        )
        self.client.get(self.url)

    def test_deleted_model(self):
        self.assertEquals(
            Page.objects.filter(slug=self.instance_slug).count(),
            0
        )

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class SecretEntryFormSetViewTest(AdminTestCase):

    def setUp(self):
        self.login_as_admin()
        self.url = reverse(
            "evoteadmin:secretentry"
        )

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response,
            "evoteadmin/secretentry.html"
        )

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)
