from __future__ import unicode_literals

from django.core.urlresolvers import reverse_lazy
from django.contrib.auth.decorators import login_required
from django.contrib.messages.views import SuccessMessageMixin
from django.utils.translation import ugettext_lazy as _
from django.utils.decorators import method_decorator
# from django.utils.translation import ugettext_lazy as _
from django.views.generic import TemplateView, ListView, DetailView
from django.views.generic.edit import UpdateView, DeleteView

# from django_tables2 import SingleTableMixin
from evotecore.mixins import DjangoTablesFilterMixin
from evoteusers.views import EvoteLoginView

from .models import Page, SecretEntry
from .forms import (AdminAuthenticationForm, PageForm,
                    SecretEntryFormset)
from .tables import PageTable

from evoteajaxforms.views import FormsetJsonSimpleView
from .filters import PageListFilter


class TitledViewMixin(object):

    title = None

    def get_context_data(self, **kwargs):
        context = super(TitledViewMixin, self).get_context_data(**kwargs)
        if self.title:
            context.update({
                'title': self.title
            })
        return context


class LoginView(EvoteLoginView):
    """Provides user the ability to login"""
    form_class = AdminAuthenticationForm
    template_name = "evoteadmin/login.html"


@method_decorator(login_required, name="dispatch")
class HomepageView(TitledViewMixin, TemplateView):
    template_name = "evoteadmin/index.html"


@method_decorator(login_required, name="dispatch")
class PageListView(TitledViewMixin, DjangoTablesFilterMixin, ListView):
    model = Page
    template_name = "evoteadmin/page_list.html"
    title = _("Pages")
    table_class = PageTable
    table_pagination = {'per_page': 5}
    filter_class = PageListFilter


@method_decorator(login_required, name="dispatch")
class PageUpdateView(TitledViewMixin, SuccessMessageMixin, UpdateView):
    model = Page
    form_class = PageForm
    template_name_suffix = '_update'
    title = _("Update Page")
    success_message = _("Page updated successfully")
    success_url = reverse_lazy('evoteadmin:page-list')


@method_decorator(login_required, name="dispatch")
class PageDeleteView(TitledViewMixin, DeleteView):
    model = Page
    success_url = reverse_lazy('evoteadmin:page-list')
    title = _("Delete Page")

    # Submit delete request without confirmation. This generates a 302 response
    def get(self, *args, **kwargs):
        return self.post(*args, **kwargs)


class PageDetailView(DetailView):
    model = Page
    template_name = "voters/page_detail.html"


@method_decorator(login_required, name="dispatch")
class PagePreviewView(TitledViewMixin, DetailView):
    model = Page
    template_name = "evoteadmin/page_preview.html"
    title = _("Preview")


@method_decorator(login_required, name="dispatch")
class SecretEntryFormSetView(TitledViewMixin, FormsetJsonSimpleView):

    def get(self, request):

        # 1st. Iterate over the Identifier choices
        #   -> Try to load an already created SecretEntry.
        #   -> Create an empty one if it does not exist.
        for secret_entry_type in SecretEntry.Identifier.choices:
            try:
                SecretEntry.objects.get(identifier=secret_entry_type[0])
            except SecretEntry.DoesNotExist:
                SecretEntry.objects.create(
                    identifier=secret_entry_type[0],
                    user=request.user
                )

        return super(SecretEntryFormSetView, self).get(request)

    # TODO
    # permission_required = '???'
    template_name = "evoteadmin/secretentry.html"
    form_class = SecretEntryFormset
    queryset = SecretEntry.objects.all()
    prefix = 'secretentry'
    title = _("Certificate Upload")
    success_message = _("Certificates updated successfully")
    error_message = _("There was a problem updating Certificates")
    success_url = reverse_lazy('evoteadmin:secretentry')
