from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.utils import timezone

from guardian.shortcuts import get_objects_for_user

from .tasks import (begin_election_proposition, end_election_proposition,
                    proposition_start_notification,
                    proposition_end_notification, voting_begin_notification,
                    minute_end_notification, roll_begin_notification,
                    roll_end_notification)

from evotecore.utils import compare_data


class ElectionQuerySet(models.QuerySet):

    def closed(self):
        """Return elections on state CLOSED"""
        return self.filter(state=self.model.State.CLOSED)

    def in_to_vote(self):
        """Return elections on state IN_TO_VOTE"""
        return self.filter(state=self.model.State.IN_TO_VOTE)

    def eligible(self, user):
        """Return elections eligible for the user"""
        return get_objects_for_user(
            user, "vote_election", self.all(), use_groups=False,
            with_superuser=False)


class ElectoralRollQuerySet(models.QuerySet):

    def get_published_of_process(self):
        """Get published electoral roll on the process"""
        return self.filter(published_on__isnull=False).first()

    def get_published(self, election_id):
        """Get published electoral roll on the election"""
        return self.filter(
            election_id=election_id, published_on__isnull=False).first()

    def get_to_publish(self, election_id):
        """Get electoral roll that can be published"""
        to_publish = self.filter(
            election_id=election_id, published_on__isnull=True, imported=True)
        published = self.get_published(election_id)
        if published:
            to_publish = to_publish.filter(created__gt=published.created)
        return to_publish.first()

    def is_importing(self, election_id):
        """ """
        return self.filter(election_id=election_id, imported=None)


class CandidateQuerySet(models.QuerySet):

    def write_in(self):
        """ """
        return self.filter(write_in=True).order_by("designation")

    def not_write_in(self):
        """ """
        return self.filter(write_in=False)


class CandidateBaseManager(models.Manager):
    """ """

    def bulk_import(self, election, data):
        """ """
        candidates = []
        candidates_to_create = []
        for row in data:
            try:
                candidate = self.get(election=election,
                                     personnel_number=row['personnel_number'])
                self.efficient_update(candidate, row, [
                    "designation"
                ])
                candidates.append(candidate)
            except self.model.DoesNotExist:
                candidates_to_create.append(self.model(
                    write_in=True,
                    election=election,
                    designation=row["designation"],
                    personnel_number=row['personnel_number']
                ))
        candidates += self.bulk_create(candidates_to_create)
        # before write in imported candidates cleared
        candidate_ids = [c.pk for c in candidates]
        self.write_in().filter(election=election)\
            .exclude(pk__in=candidate_ids).delete()
        return candidates

    def efficient_update(self, voter, data, keys):
        """Only update if data changed, otherwise do nothing"""
        need_update = not compare_data(voter, data, keys)
        if need_update:
            for key in keys:
                setattr(voter, key, data[key])
            voter.save(update_fields=keys)


CandidateManager = CandidateBaseManager.from_queryset(CandidateQuerySet)


class ElectionTaskManager(object):

    def get_notifications_dates(self, eta, minute=0, second=0):
        dates = []
        days = getattr(
            settings, "EVOTE_ELECTION_NOTIFICATION_DAYS_BEFORE", [3, 1])
        hour = getattr(settings, "EVOTE_ELECTION_NOTIFICATION_HOUR", 7)
        for day in days:
            noti_date = eta - timezone.timedelta(days=day)
            noti_date = noti_date.replace(
                hour=hour, minute=minute, second=second)
            dates.append((noti_date, day))
        return dates

    def begin_election_proposition_task(self, election_id, eta):
        noti_dates = self.get_notifications_dates(eta)
        for date, days in noti_dates:
            proposition_start_notification.si(election_id, days).apply_async(
                eta=date)

        begin_election_proposition.si(election_id).apply_async(eta=eta)

    def end_election_proposition_task(self, election_id, eta):
        noti_dates = self.get_notifications_dates(eta)
        for date, days in noti_dates:
            proposition_end_notification.si(election_id, days).apply_async(
                eta=date)

        end_election_proposition.si(election_id).apply_async(eta=eta)

    def begin_election_voting_task(self, election_id, eta):
        noti_dates = self.get_notifications_dates(eta)
        for date, days in noti_dates:
            voting_begin_notification.si(election_id, days).apply_async(
                eta=date)

    def end_election_minute_task(self, election_id, eta):
        noti_dates = self.get_notifications_dates(eta)
        for date, days in noti_dates:
            minute_end_notification.si(election_id, days).apply_async(
                eta=date)

    def begin_election_roll_task(self, election_id, eta):
        noti_dates = self.get_notifications_dates(eta)
        for date, days in noti_dates:
            roll_begin_notification.si(election_id, days).apply_async(
                eta=date)

    def end_election_roll_task(self, election_id, eta):
        noti_dates = self.get_notifications_dates(eta)
        for date, days in noti_dates:
            roll_end_notification.si(election_id, days).apply_async(
                eta=date)


election_task_manager = ElectionTaskManager()
