from __future__ import unicode_literals

from datetime import timedelta

from django.db.models.signals import post_save, post_init, pre_save
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible
from django.utils import timezone
from django.conf import settings
from django.db import models

from evotevotings.processes.models import ElectoralProcess
from django_extensions.db.models import TimeStampedModel
from evotecore.mixins import ModelFieldChangedMixin
from evotecore.decorators import searchable
from evotedocs.models import Document
from evotecore.utils import images_upload_to
from sorl.thumbnail import ImageField
from django_fsm import FSMIntegerField, transition
from autoslug import AutoSlugField

from .managers import (ElectionQuerySet, ElectoralRollQuerySet,
                       CandidateManager, election_task_manager)

from evotevotings.services.routers import ASrouter
from .utils import initialize_blind_signatures


class ElectionCalendarMixin(models.Model):
    """Mixin with electoral calendar for code responsability separation
    proposes
    """
    proposition_begin_date = models.DateTimeField(
        _("begin date"))
    proposition_end_date = models.DateTimeField(
        _("end date"))
    roll_begin_date = models.DateTimeField(
        _("publication date before verification"))
    roll_end_date = models.DateTimeField(
        _("publication date after verification and fixes"))
    voting_begin_date = models.DateTimeField(
        _("begin date"))
    voting_end_date = models.DateTimeField(
        _("end date"))
    voting_references_date = models.DateTimeField(
        _("consult date"))
    minute_publication_date = models.DateTimeField(
        _("publication date"))
    minute_begin_date = models.DateTimeField(
        _("verification begin date"))
    minute_end_date = models.DateTimeField(
        _("verification end date"))

    class Meta:
        abstract = True


class BulletinMixin(models.Model):
    """Mixin with bulletin info for code responsability separation proposes
    """

    class PresentationMode:
        LIST = 0
        DROPDOWN = 1

        choices = (
            (LIST, _("List")),
            (DROPDOWN, _("Dropdown")),
        )

    presentation_mode = models.PositiveSmallIntegerField(
        _("presentation mode"), choices=PresentationMode.choices,
        default=PresentationMode.LIST)
    instructions = models.CharField(_("instructions"), max_length=256,
                                    blank=True)

    class Meta:
        abstract = True

    def is_list_mode(self):
        return self.presentation_mode == self.PresentationMode.LIST

    def is_dropdown_mode(self):
        return self.presentation_mode == self.PresentationMode.DROPDOWN


class ElectionManageMixin(models.Model):
    """ """
    configs_filled = models.BooleanField(editable=False, default=False)
    calendar_filled = models.BooleanField(editable=False, default=False)
    bulletin_filled = models.BooleanField(editable=False, default=False)

    class Meta:
        abstract = True


class ElectionCriptoMixin(models.Model):
    """ Mixin to extend the Election with criptography
        related fields"""
    certificate = models.TextField(_("election certificate"), null=True)
    cert_private_key = models.TextField(
        _("election certificate private key"), null=True)
    blindsig_pr_dash_components = models.TextField(
        _("blind signature pr dash components"),
        editable=False, default="")
    blindsig_init_components = models.TextField(
        _("blind signature init components"),
        editable=False, default="")

    class Meta:
        abstract = True


@searchable(title='identifier', content='description')
@python_2_unicode_compatible
class Election(
        ModelFieldChangedMixin, BulletinMixin, ElectionManageMixin,
        ElectionCalendarMixin, ElectionCriptoMixin, TimeStampedModel):
    """
    TODO: define REMARKABLE_CHOICES and HONDT_CHOICES
    """

    class State:
        NEW = 0
        OPEN = 1
        PROVISIONAL_ELECTORAL_ROLL = 2
        DEFINITIVE_ELECTORAL_ROLL = 3
        IN_TO_VOTE = 4
        IN_COUNTING = 5
        VOTES_SCRUTINIZED = 6
        MINUTE_IN_APPROVAL = 7
        CLOSED = 8

        choices = (
            (NEW, _("New")),
            (OPEN, _("Open")),
            (PROVISIONAL_ELECTORAL_ROLL, _("Provisional electoral roll")),
            (DEFINITIVE_ELECTORAL_ROLL, _("Definitive electoral roll")),
            (IN_TO_VOTE, _("In to vote")),
            (IN_COUNTING, _("In counting")),
            (VOTES_SCRUTINIZED, _("Votes scrutinized")),
            (MINUTE_IN_APPROVAL, _("Minute in approval")),
            (CLOSED, _("Closed")),
        )

    REMARKABLE_CHOICES = [(n, n) for n in range(1, 10)]
    HONDT_CHOICES = [(n, n) for n in range(1, 10)]

    electoral_process = models.ForeignKey(
        ElectoralProcess, verbose_name=_("electoral process"),
        on_delete=models.CASCADE, related_name='elections')
    identifier = models.CharField(
        _("election identifier"), max_length=128,
        help_text=_("Insert the election designation"))
    slug = AutoSlugField(
        populate_from='identifier', unique=True, editable=False)
    description = models.CharField(
        _("election description"), max_length=512,
        help_text=_("brief description text about election"))
    state = FSMIntegerField(
        _("state"), choices=State.choices, default=State.NEW)
    limit_remarkable_choices = models.PositiveSmallIntegerField(
        _("limit remarkable choices"), choices=REMARKABLE_CHOICES, default=1)
    write_in = models.BooleanField(_("write in?"), default=True)
    write_in_import = models.ForeignKey(
        Document, verbose_name=_("import file"), related_name="+", null=True)
    ordered_voting = models.BooleanField(_("ordered voting?"), default=True)
    weight_votes = models.BooleanField(_("weight votes?"), default=True)
    hondt_method = models.BooleanField(_("hondt method?"), default=True)
    hondt_total_seats = models.PositiveSmallIntegerField(
        _("hondt total seats"), choices=HONDT_CHOICES, null=True)

    objects = ElectionQuerySet.as_manager()

    class Meta:
        verbose_name = _("election")
        verbose_name_plural = _("elections")
        permissions = (
            ('vote_election', 'Vote election'),
            ('view_election', 'View election'),
            # change_election already exists
        )

    def __str__(self):
        return self.identifier

    @property
    def is_within_voting_period(self):
        """Checks if the current date is within the
           Election voting_begin_date and voting_end_date"""
        current_date = timezone.now()
        return self.voting_begin_date <= current_date <= self.voting_end_date

    @property
    def is_in_definitive_eletoral_roll(self):
        return self.state == self.State.DEFINITIVE_ELECTORAL_ROLL

    @property
    def is_in_to_vote(self):
        return self.state == self.State.IN_TO_VOTE

    @property
    def can_scrutinize(self):
        return self.state >= self.State.IN_COUNTING

    def can_publish(self):
        return self.electoral_process.can_publish()

    def can_begin_proposition(self):
        return self.proposition_begin_date <= timezone.now()

    def can_begin_voting(self):
        return self.voting_begin_date <= timezone.now()

    def can_end_voting(self):
        return self.voting_end_date <= timezone.now()

    def can_end_proposition(self):
        return self.proposition_end_date <= timezone.now()

    def can_close(self):
        return self.minute_end_date <= timezone.now()

    @transition(state, source=State.NEW, target=State.OPEN,
                conditions=[can_publish])
    def publish(self):
        """Publish the election accordingly to the electoral process
        publication date. This action is triggered by the electoral process
        """

    @transition(state, source=State.OPEN,
                target=State.PROVISIONAL_ELECTORAL_ROLL,
                conditions=[can_begin_proposition])
    def begin_proposition(self):
        """Transition only called when proposition_begin_date is reached
        """

    @transition(state, source=State.PROVISIONAL_ELECTORAL_ROLL,
                target=State.DEFINITIVE_ELECTORAL_ROLL,
                conditions=[can_end_proposition])
    def end_proposition(self):
        """Transition only called when proposition_end_date is reached
        """
        try:
            # Get the final electoral roll cardinality
            voters = self.election_voters.count()
            # Connect to the Anonymizer Service
            as_router = ASrouter()
            # TODO: Handle timeout requests
            as_router.initialize_election(self.pk,
                                          voters)

        except Exception as e:
            # TODO: Log
            raise e

    @transition(state, source=State.DEFINITIVE_ELECTORAL_ROLL,
                target=State.IN_TO_VOTE, conditions=[can_begin_voting])
    def voting(self):
        """Transition to a in to vote state"""

    @transition(state, source=State.IN_TO_VOTE, target=State.IN_COUNTING,
                conditions=[can_end_voting])
    def counting(self):
        """Transition to a in counting state"""

    @transition(state, source=State.IN_COUNTING,
                target=State.VOTES_SCRUTINIZED)
    def scrutinized(self):
        """Transition to votes scrutinized"""

    @transition(state, source=State.VOTES_SCRUTINIZED,
                target=State.MINUTE_IN_APPROVAL)
    def minute_in_approval(self):
        """Transition to minute in approval"""

    @transition(state, source=State.MINUTE_IN_APPROVAL, target=State.CLOSED)
    def close(self):
        """Transition when minute_end_date is reached"""

    def can_manage_operations(self):
        """
        """
        # in days
        return True
        if self.voting_begin_date:
            days = getattr(settings, 'EVOTE_ELECTION_OPERATIONS_AFTER', 3)
            delta = timezone.now() + timedelta(days=days)
            return delta >= self.voting_begin_date
        return False

    def can_change_configs(self):
        """Only can change bulletin info if actions have been configured
        """
        return self.electoral_process.actions_configured

    def can_change_calendar(self):
        """Only can change bulletin info if actions have been configured
        """
        return self.electoral_process.actions_configured

    def can_change_categories(self):
        """Only can change bulletin info if actions have been configured
        """
        return self.electoral_process.actions_configured

    def can_change_rolls(self):
        """Only can change bulletin info if actions have been configured
        """
        return self.electoral_process.actions_configured

    def can_change_candidates(self):
        """Only can change bulletin info if actions have been configured
        and proposition_end_date have been reached"""
        if self.proposition_end_date:
            return self.electoral_process.actions_configured and \
                timezone.now() >= self.proposition_end_date
        return False

    def can_change_bulletin(self):
        """Only can change bulletin info if actions have been configured,
        configs has been filled and candidates created"""
        return self.electoral_process.actions_configured and \
            self.configs_filled and self.candidates.exists()

    @staticmethod
    def post_init(sender, instance, **kwargs):
        """Callback staticmethod for post_init signal"""
        instance.track_changes([
            'proposition_begin_date', 'proposition_end_date',
            'minute_end_date', 'voting_begin_date', 'roll_begin_date',
            'roll_end_date'
        ])

    @staticmethod
    def pre_save(sender, instance, **kwargs):
        """Callback staticmethod for pre_save signal"""
        # On creating the object, initialize
        # the blind signature components
        if instance._state.adding is True:
            # Creating the object
            (init_component, pr_component) = initialize_blind_signatures()
            instance.blindsig_pr_dash_components = pr_component
            instance.blindsig_init_components = init_component

    @staticmethod
    def post_save_changes(sender, instance, **kwargs):
        """Callback staticmethod for post_save signal"""
        if instance.has_changed('proposition_begin_date'):
            election_task_manager.begin_election_proposition_task(
                instance.pk, eta=instance.proposition_begin_date)
        if instance.has_changed('proposition_end_date'):
            election_task_manager.end_election_proposition_task(
                instance.pk, eta=instance.proposition_end_date)
        if instance.has_changed('minute_end_date'):
            election_task_manager.end_election_minute_task(
                instance.pk, eta=instance.minute_end_date)
        if instance.has_changed('voting_begin_date'):
            election_task_manager.begin_election_voting_task(
                instance.pk, eta=instance.voting_begin_date)
        if instance.has_changed('roll_begin_date'):
            election_task_manager.begin_election_roll_task(
                instance.pk, eta=instance.roll_begin_date)
        if instance.has_changed('roll_end_date'):
            election_task_manager.end_election_roll_task(
                instance.pk, eta=instance.roll_end_date)

    @staticmethod
    def post_save_create_category(sender, instance, **kwargs):
        """create a default category"""
        if kwargs.get('created'):
            # TODO: pass designations to settings.
            instance.categories.create(
                designation_en="One", designation_pt="Um")


post_init.connect(Election.post_init, sender=Election)
pre_save.connect(Election.pre_save, sender=Election)
post_save.connect(Election.post_save_changes, sender=Election)
post_save.connect(Election.post_save_create_category, sender=Election)


def populate_electoral_roll(instance):
    return instance.document.designation


@python_2_unicode_compatible
class ElectoralRoll(TimeStampedModel):
    """ """
    election = models.ForeignKey(
        Election, verbose_name=_("election"), related_name="electoral_rolls",
        on_delete=models.CASCADE)
    document = models.ForeignKey(
        Document, verbose_name=_("document"), related_name="+")
    slug = AutoSlugField(
        populate_from=populate_electoral_roll,
        unique=True, editable=False)
    published_on = models.DateTimeField(
        _("published on"), null=True, default=None)
    imported = models.NullBooleanField(default=None)
    objects = ElectoralRollQuerySet.as_manager()

    class Meta:
        ordering = ("-created",)
        verbose_name = _("electoral roll")
        verbose_name_plural = _("electoral rolls")

    def __str__(self):
        return self.document.designation

    @property
    def election_voters(self):
        return self.election.election_voters.all()

    @property
    def version(self):
        return self.created.strftime('%Y%m%d%H%M')

    @property
    def published(self):
        return bool(self.published_on)


@python_2_unicode_compatible
class Category(TimeStampedModel):
    """ """
    election = models.ForeignKey(
        Election, verbose_name=_("election"), related_name="categories",
        on_delete=models.CASCADE)
    designation = models.CharField(_("category designation"), max_length=128)
    slug = AutoSlugField(
        populate_from='designation', unique=True, editable=False)
    weight = models.PositiveSmallIntegerField(_("category weight"), default=1)

    class Meta:
        verbose_name = _("category")
        verbose_name_plural = _("categories")

    def __str__(self):
        return self.designation


@python_2_unicode_compatible
class Candidate(TimeStampedModel):
    """ """
    election = models.ForeignKey(
        Election, verbose_name=_("election"), related_name="candidates",
        on_delete=models.CASCADE)
    logo = ImageField(_("logo"), upload_to=images_upload_to, blank=True)
    personnel_number = models.CharField(_('personnel number'), max_length=64,
                                        db_index=True)
    designation = models.CharField(_("designation"), max_length=128)
    documents = models.ManyToManyField(Document, verbose_name=_("documents"),
                                       blank=True)
    write_in = models.BooleanField(default=False)
    order = models.PositiveSmallIntegerField(_("order"), default=0)
    objects = CandidateManager()

    class Meta:
        ordering = ("order",)
        verbose_name = _("candidate")
        verbose_name_plural = _("candidates")

    def __str__(self):
        return self.designation


@python_2_unicode_compatible
class ElectionCountQuery(TimeStampedModel):
    """ """
    election = models.ForeignKey(
        Election, verbose_name=_("election"), related_name="count_queries",
        on_delete=models.CASCADE)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_("user"),
        on_delete=models.CASCADE, related_name="+")
    total_count = models.PositiveSmallIntegerField(_("Total count"), null=True)

    class Meta:
        ordering = ("-created",)
        verbose_name = _("Count query")
        verbose_name_plural = _("Count queries")

    def __str__(self):
        return "{} ({}) - {}".format(self.election, self.user.name,
                                     self.total_count)
