import json
from mock import patch
from unittest import skip

from django.test import TestCase
from django.utils import timezone
from django.core.urlresolvers import reverse_lazy

from evotevotings.elections.utils import (get_election_data,
                                          get_extra_election_data)
from evotevotings.admin.utils import (get_certificates,
                                      not_expired_certificate,
                                      validate_certificate,
                                      get_secret_entry,
                                      get_private_key_password)
from evotevotings.services.validator.tests.utils import (
    ValidatorServiceTestUtils)
from evotevotings.elections.tests.utils import ElectionsTestUtils
from evotevotings.admin.models import SecretEntry
from eVotUM.Cripto import pkiutils, eccblind
from evotevotings.voters.models import ElectionVoter


# Helper Class
class ServiceTestUtils(ElectionsTestUtils):
    """ Helper functions class """

    def generate_fake_election(self):
        now = timezone.now()
        vbegin = now - timezone.timedelta(days=1)
        vend = now + timezone.timedelta(days=1)
        # Generate fixtures
        votesys_cert_id = SecretEntry.Identifier.VOTING_CERT
        self.election = self.create_election(
            voting_begin_date=vbegin, voting_end_date=vend,
            certificate=get_secret_entry(votesys_cert_id))
        self.create_candidates(4,
                               election=self.election)
        self.voter = self.create_voter(
            election=self.election
        )
        self.election_data = self.fake_election_data(self.election.pk)
        self.certificates = get_certificates()
        # Initialize the blind signature
        (init_component, pr_component) = eccblind.initSigner()
        self.init_component = init_component
        self.pr_component = pr_component

    def fake_election_data(self, election_id):
        return get_election_data(election_id)


class ValidCertificateTestCase(TestCase):
    fixtures = ['users.json', 'certs.json']


class ExpiredRootCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""

    fixtures = ['users.json', 'certs_exp_root.json']
    # Setup the Test Case

    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        root_cert = self.certificates['root']
        expired = not_expired_certificate(root_cert)
        self.assertFalse(expired)


class ExpiredVotingCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""

    fixtures = ['users.json', 'certs_exp_voting.json']

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        cert = self.certificates['votesys']
        expired = not_expired_certificate(cert)
        self.assertFalse(expired)


class UnchainedVotingCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""

    fixtures = ['users.json', 'certs_unchained_voting.json']

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        cert = self.certificates['votesys']
        root_cert = self.certificates['root']
        chain = validate_certificate(cert, root_cert)
        self.assertEqual(chain, -1)


class ExpiredValidatorCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""

    fixtures = ['users.json', 'certs_exp_vs.json']

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        cert = self.certificates['vs']
        expired = not_expired_certificate(cert)
        self.assertFalse(expired)


class UnchainedValidorCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""

    fixtures = ['users.json', 'certs_unchained_vs.json']

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        cert = self.certificates['vs']
        root_cert = self.certificates['root']
        chain = validate_certificate(cert, root_cert)
        self.assertEqual(chain, -1)


class StepZeroTest(ValidCertificateTestCase, ServiceTestUtils):
    """ Class to test the services workflow Step 0"""

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    # Test Units
    def test_dataintegrity(self):
        """ Check if election data was properly loaded """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.pk)
        self.assertTrue(extra_data)

    def test_invalid_election_id(self):
        """ Check unexpected election id """
        extra_data = get_extra_election_data(-1,
                                             self.voter.pk)
        self.assertFalse(extra_data)

    @patch("evotevotings.elections.utils.get_secret_entry", return_value=False)
    def test_invalid_certificate(self, mock_fn):
        """ Check for invalid certificates """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.pk)
        self.assertFalse(extra_data)

    def test_valid_certificate(self):
        """ Check if election certificate is valid """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.pk)
        cert = extra_data['vs_cert']
        self.assertTrue(pkiutils.verifyPEMCertificate(cert))

    @patch("evotevotings.elections.utils.get_secret_entry", return_value=False)
    def test_invalid_private_key(self, mock_fn):
        """ Check for invalid certificates """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.pk)
        self.assertFalse(extra_data)

    @patch("evotevotings.elections.utils.pkiutils.signObject",
           return_value=(1, None))
    def test_unsinged_objects(self, mock_fn):
        """ Check for unsigned object """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.pk)
        self.assertFalse(extra_data)


class StepTwoTest(ValidCertificateTestCase,
                  ServiceTestUtils, ValidatorServiceTestUtils):
    """ Class to test the services workflow Step 2"""

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

        # Validate signature (test_validate_signature)

        # Validate Chain certificate (test_system_certificate)

        # Validate Voting System certificate
        # (test_valid_voting_system_certificate)

        # Validate Root certificate (test_valid_root_certificate)

        # Validate Election period (test_election_period)

        # Validate ElectionVoter permission to vote
        # (test_voter_permission)

        # Validate Voting Status (test_voter_status)

        # Sign the blind vote (test_blind_signature_signin)

    def test_system_certificate(self):
        """ Check if voting system certificate was issued
            with chain root certificate """
        election_id = self.election.id
        certs = get_certificates(election_id)
        vote_system_cert = certs['votesys']
        root_cert = certs['root']
        chain = pkiutils.verifyCertificateChain(
            vote_system_cert,
            root_cert
        )

        self.assertTrue(chain)

    def test_valid_root_certificate(self):
        """ Check if root certificate is within validation period """
        root_cert = self.certificates['root']
        not_expired = not_expired_certificate(root_cert)
        self.assertTrue(not_expired)

    def test_valid_voting_system_certificate(self):
        """ Check if certificate is within validation period """
        vote_system_cert = self.certificates['votesys']
        not_expired = not_expired_certificate(vote_system_cert)
        self.assertTrue(not_expired)

    def test_election_period(self):
        """ Check if election is in voting period """
        self.assertTrue(self.election.is_within_voting_period)

    def test_voter_permission(self):
        """ Check if election voter can vote in this election """
        self.assertIs(self.voter.election, self.election)

    def test_voter_status(self):
        """ Check if vote is available for the voter in the election"""
        self.assertTrue(self.voter.is_vote_available[1])

    def test_validate_signature(self):
        """ Check if the user + election signature is valid"""
        signature = self.election_data['combo_id']
        votesys_cert = self.certificates['votesys']
        (error_code, obj) = pkiutils.verifyObjectSignature(
            signature,
            votesys_cert
        )
        self.assertIsNone(error_code)


class StepFourAPITest(ValidCertificateTestCase,
                      ServiceTestUtils, ValidatorServiceTestUtils):
    """ Class to test the services workflow Step 4"""
    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()
        self.url = reverse_lazy('genericservice:filtersignature')

    @skip('Merge problem to fix')
    def generate_fake_request_data(self):
        ciphered_vote = 'testcpheredvote'
        sha_cv = ("4b12333d0a1a5f55592678646c6b1e50b"
                  "4dd7ae9e37c7988a6b2d8a65fe8bb9b")

        vs_private_key = get_secret_entry(
            SecretEntry.Identifier.VALIDATOR_PVT_KEY)
        pvtkey_password = get_private_key_password()

        errorCode, signature = self.get_blind_signature(vs_private_key,
                                                        pvtkey_password,
                                                        sha_cv,
                                                        self.pr_component,
                                                        self.init_component)

        data = {'ciphered_vote': ciphered_vote,
                'signed_vote': signature,
                'election_id': self.election_data['election_id'],
                'unique_bulletin': self.election_data['bulletin'],
                'blind_components': self.init_component,
                'pr_components': self.pr_component,
                'blind_hash': sha_cv}

        return data

    @skip('Merge problem to fix')
    def test_api_call_fail(self):
        r = self.client.post(self.url)
        self.assertEqual(r.status_code, 400)

    @skip('Merge problem to fix')
    def test_api_call_false(self):
        data = self.generate_fake_request_data()
        r = self.client.post(self.url, data)
        self.assertEqual(r.status_code, 200)
        json_resp = r.content.decode('string-escape').strip('"')
        response = json.loads(json_resp)
        self.assertEquals(response['result'], "false")

    @skip('Merge problem to fix')
    def test_api_call_ok(self):
        data = self.generate_fake_request_data()
        self.voter.vote_status = ElectionVoter.VoteStatus.PENDING
        self.voter.blind_vote_hash = data['blind_hash']
        self.voter.save()
        r = self.client.post(self.url, data)
        self.assertEqual(r.status_code, 200)
        json_resp = r.content.decode('string-escape').strip('"')
        response = json.loads(json_resp)
        self.assertEquals(response['result'], "true")
        self.voter.refresh_from_db()
        self.assertEqual(self.voter.vote_status,
                         ElectionVoter.VoteStatus.YES)
