from __future__ import unicode_literals

from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.shortcuts import get_object_or_404
from django.views import View
from django.http import JsonResponse, Http404

from evotevotings.processes.models import ElectoralProcess, ElectoralAction
from evotevotings.management.forms import ElectoralMemberEmailForm
from evotevotings.elections.models import Election, ElectionCountQuery
from evoteajaxforms.views import FormPartialJsonView, DetailPartialJsonView
from evotemessages.ajax import MessageRecipientsAjaxView
from django_tables2 import SingleTableMixin

from .registers import action_validation
from .tables import ElectionCountQueryTable
from .forms import ActionValidationKeysForm


@method_decorator(login_required, name="dispatch")
class ActionValidationAjaxView(FormPartialJsonView):
    """ """
    template_name = "management/partials/action_validate.html"
    form_class = ActionValidationKeysForm

    def get_object(self):
        """ """
        return get_object_or_404(
            ElectoralAction, pk=self.kwargs.get("pk", None),
            electoral_process=self.kwargs.get("process_pk"))

    def get_action_identifier(self):
        return self.kwargs.get("pk")

    def get_form_kwargs(self):
        """ """
        kwargs = super(ActionValidationAjaxView, self).get_form_kwargs()
        electoral_action = self.get_object()
        electoral_process = electoral_action.electoral_process
        kwargs.update({
            'num_members': electoral_process.electoral_members.count(),
            'strength': electoral_action.strength
        })
        return kwargs

    def form_valid(self, form):
        action_validation.register(
            self.request, self.kwargs.get("process_pk"),
            self.get_action_identifier())
        return super(ActionValidationAjaxView, self).form_valid(form)


@method_decorator(login_required, name="dispatch")
class ManageActionValidationAjaxView(ActionValidationAjaxView):
    """ """

    def get_form_kwargs(self):
        """ """
        electoral_process = get_object_or_404(
            ElectoralProcess, pk=self.kwargs.get("process_pk"))
        kwargs = super(ActionValidationAjaxView, self).get_form_kwargs()
        kwargs.update({
            'num_members': electoral_process.electoral_members.count(),
            'strength': ElectoralAction.Strength.CRITICAL
        })
        return kwargs

    def get_action_identifier(self):
        return "manage"


@method_decorator(login_required, name="dispatch")
class ElectoralMemberVerifyView(View):
    """ """

    def get(self, request, *args, **kwargs):
        form = ElectoralMemberEmailForm(request.GET,
                                        process_pk=kwargs['process_pk'])

        if form.is_valid():
            user = form.get_user()

            if user:
                response_data = {
                    'name': user['name'],
                    'phone': str(user['phone']),
                    'enable': ['role', 'designation']
                }
        else:
            response_data = {
                'error': form.errors['email']
            }

        return JsonResponse(response_data, status=200)


@method_decorator(login_required, name="dispatch")
class ElectionCountingsPartialView(SingleTableMixin, DetailPartialJsonView):
    """ """
    model = Election
    table_class = ElectionCountQueryTable
    template_name = "management/partials/election_countings.html"

    def dispatch(self, *args, **kwargs):
        """Raise 404 when elections is not within voting period"""
        self.object = self.get_object()
        if not self.object.is_within_voting_period:
            raise Http404
        return super(ElectionCountingsPartialView, self)\
            .dispatch(*args, **kwargs)

    def get_table_data(self):
        """Return count queries queyset for this election"""
        return self.object.count_queries.all()

    def get_context_data(self, **kwargs):
        """Call api for total votes and add the value to the context"""
        # must be called before super() to include the new registered count
        total_votes = self.get_total_votes(self.request.user)
        context = super(ElectionCountingsPartialView, self)\
            .get_context_data(**kwargs)
        context.update({
            'total_votes': total_votes
        })
        return context

    def get_total_votes(self, user):
        """Request a votes counting and register the request"""
        # TODO: make request to get the total votes
        total_votes = 20000
        ElectionCountQuery.objects.create(
            election=self.object,
            user=user,
            total_count=total_votes
        )
        return total_votes


@method_decorator(login_required, name="dispatch")
class ElectionScrutinyStatusView(View):
    """ """

    def get(self, request, *args, **kwargs):
        get_object_or_404(
            Election, pk=kwargs.get('pk'),
            electoral_process_id=kwargs.get('process_pk'))
        # TODO: replace with service method
        data = {
            "status": 4
        }
        return JsonResponse(data, status=200)


class MessageRecipientsManageAjaxView(MessageRecipientsAjaxView):
    """ """
