from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from django.core.exceptions import ValidationError
from django.forms import (BaseInlineFormSet, inlineformset_factory,
                          modelformset_factory)
from django.utils import timezone
from django import forms

from evotevotings.elections.importers import (ElectoralRollImporter,
                                              CandidateImporter)
from evotevotings.elections.models import Election, Category, Candidate
from evotevotings.processes.models import (ElectoralProcess, ElectoralAction,
                                           ElectoralMember, ElectoralDocument)
from evotevotings.elections.forms import ElectoralCalendarForm
from evoteusers.forms import EvoteAuthenticationForm
from evotecore import widgets
from evotecore.mixins import NestedFormsetFormMixin
from evotedocs.models import Document
from evotedocs.forms import (DocumentForm, DocumentFormset,
                             OrderedDocumentFormset)

from contrib.umws.members.members_service import members_service


class ManagementAuthenticationForm(EvoteAuthenticationForm):
    """ """

    class Profiles:
        ELECTORAL_MEMBER = 0
        INSTITUTIONAL_RESPONSIBLE = 1

        choices = (
            (ELECTORAL_MEMBER, _("Electoral Member")),
            (INSTITUTIONAL_RESPONSIBLE, _("Institutional Responsible")),
        )

    profile = forms.TypedChoiceField(
        label=_("Profile"), choices=Profiles.choices, coerce=int,
        initial=Profiles.ELECTORAL_MEMBER, widget=widgets.Select)

    def confirm_login_allowed(self, user, cleaned_data):
        super(ManagementAuthenticationForm, self).confirm_login_allowed(user)
        is_institutional_responsible = cleaned_data.get('profile') == \
            self.Profiles.INSTITUTIONAL_RESPONSIBLE and \
            user.is_institutional_responsible
        is_electoral_member = cleaned_data.get('profile') == \
            self.Profiles.ELECTORAL_MEMBER and user.electoral_members.exists()
        if not is_institutional_responsible and not is_electoral_member:
            raise forms.ValidationError(
                self.error_messages['invalid_login'],
                code='invalid_login',
            )


class ActionValidationKeysForm(forms.Form):
    """Form to introduce all keys for action security validation.
    The keys shown by the form are dinamic, taking in account the number of
    electoral """

    def __init__(self, *args, **kwargs):
        self.num_members = kwargs.pop("num_members")
        self.strength = kwargs.pop("strength")
        super(ActionValidationKeysForm, self).__init__(*args, **kwargs)
        # by default form has already a key field
        for num in range(1, (self.num_keys + 1)):
            self.fields["key%d" % num] = forms.FileField(
                label=_("Key %d" % num))

    @property
    def num_keys(self):
        if self.strength == ElectoralAction.Strength.SIMPLE:
            return 1
        elif self.strength == ElectoralAction.Strength.STRONG:
            return (self.num_members / 2) + 1
        return self.num_members


ElectoralActionFormset = inlineformset_factory(
    ElectoralProcess, ElectoralAction, fk_name="electoral_process", extra=0,
    can_delete=False, fields=("action", "strength"), widgets={
        'action': forms.HiddenInput(),
        'strength': forms.RadioSelect()
    })


class ElectoralProcessBaseForm(forms.ModelForm):
    """ """

    class Meta:
        model = ElectoralProcess
        fields = ('logo', 'identifier_pt', 'identifier_en', 'description_pt',
                  'description_en')
        widgets = {
            'description_pt': forms.Textarea(attrs={'rows': 4}),
            'description_en': forms.Textarea(attrs={'rows': 4})
        }


class ElectoralProcessAsInstitutionalForm(ElectoralProcessBaseForm):
    """ """

    def __init__(self, *args, **kwargs):
        super(ElectoralProcessAsInstitutionalForm, self).__init__(
            *args, **kwargs)
        if not self.instance.is_draft():
            for field in self.fields:
                self.fields[field].widget.attrs['disabled'] = True


class ElectoralProcessAsMemberForm(ElectoralProcessBaseForm):
    """ """

    def __init__(self, *args, **kwargs):
        super(ElectoralProcessAsMemberForm, self).__init__(*args, **kwargs)
        if not self.instance.is_draft() and not self.instance.is_new():
            self.fields['publication_date'].widget.attrs['disabled'] = True

    class Meta(ElectoralProcessBaseForm.Meta):
        fields = ElectoralProcessBaseForm.Meta.fields + ('publication_date', )
        election_widgets = ElectoralProcessBaseForm.Meta.widgets
        election_widgets.update({
            'publication_date': widgets.DateTimeInput()
        })
        widgets = election_widgets

    def clean_publication_date(self):
        if not self.instance.is_draft() and not self.instance.is_new():
            return self.instance.publication_date

        publication_date = self.cleaned_data.get('publication_date', None)
        if publication_date and publication_date >= timezone.now():
            return publication_date

        raise ValidationError(_('The publication date must be after today'))


class ElectoralDocumentForm(NestedFormsetFormMixin, forms.ModelForm):

    formset_class = DocumentFormset
    formset_prefix = "documents"

    class Meta:
        model = ElectoralDocument
        fields = ('designation',)

    def get_queryset(self):
        """Return many to many queryset if candidate already exists on db"""
        if self.instance and self.instance.pk:
            return self.instance.documents.all()
        return Document.objects.none()

    def _save_m2m(self, *args, **kwargs):
        """Save and associates documents to the instance"""
        instances = self.formset.save(*args, **kwargs)
        self.instance.documents.add(*instances)
        super(ElectoralDocumentForm, self)._save_m2m(*args, **kwargs)


ElectoralDocumentFormset = inlineformset_factory(
    ElectoralProcess, ElectoralDocument, extra=0, min_num=1,
    form=ElectoralDocumentForm)


class ElectoralProcessCommunicationForm(forms.Form):
    """ """
    responsible_communication = forms.ModelChoiceField(
        required=True, empty_label=_('No responsible selected'),
        queryset=ElectoralMember.objects.none(),
        widget=widgets.SearchableSelect)

    class Meta:
        fields = ('responsible_communication',)

    def __init__(self, *args, **kwargs):
        """" """
        electoral_process = kwargs.pop('instance')
        super(ElectoralProcessCommunicationForm, self).__init__(
            *args, **kwargs)
        self.fields['responsible_communication'].queryset = \
            electoral_process.electoral_members.all()


class ElectionForm(ElectoralCalendarForm):
    """ """

    class Meta(ElectoralCalendarForm.Meta):
        model = Election
        fields = ('id', 'identifier_pt', 'identifier_en', 'description_pt',
                  'description_en', 'proposition_begin_date',
                  'proposition_end_date', 'roll_begin_date', 'roll_end_date',
                  'voting_begin_date', 'voting_end_date',
                  'voting_references_date', 'minute_publication_date',
                  'minute_begin_date', 'minute_end_date')
        election_widgets = ElectoralCalendarForm.Meta.widgets
        election_widgets.update({
            'description_pt': forms.Textarea(attrs={'rows': 4}),
            'description_en': forms.Textarea(attrs={'rows': 4}),
            'id': forms.HiddenInput()
        })
        widgets = election_widgets


class ElectionsInlineFormSet(BaseInlineFormSet):

    def __init__(self, *args, **kwargs):
        super(ElectionsInlineFormSet, self).__init__(*args, **kwargs)
        if not self.instance.is_draft():
            for form in self.forms:
                for field in form.fields:
                    form.fields[field].widget.attrs['disabled'] = True


ElectionFormset = inlineformset_factory(
    ElectoralProcess, Election, extra=0, form=ElectionForm, min_num=1,
    formset=ElectionsInlineFormSet)


class ElectionDetailConfigsForm(forms.ModelForm):
    """ """

    class Meta:

        class YesNo:
            choices = (
                (True, _('Yes')),
                (False, _('No'))
            )

        model = Election
        fields = ('identifier_pt', 'identifier_en', 'description_pt',
                  'description_en', 'limit_remarkable_choices', 'write_in',
                  'ordered_voting', 'weight_votes', 'hondt_method',
                  'hondt_total_seats')
        widgets = {
            'description_en': forms.Textarea(attrs={'rows': 3}),
            'description_pt': forms.Textarea(attrs={'rows': 3}),
            'limit_remarkable_choices': widgets.Select(),
            'write_in': forms.RadioSelect(choices=YesNo.choices),
            'ordered_voting': forms.RadioSelect(choices=YesNo.choices),
            'weight_votes': forms.RadioSelect(choices=YesNo.choices),
            'hondt_method': forms.RadioSelect(choices=YesNo.choices),
            'hondt_total_seats': widgets.Select(),
        }


CategoryFormset = inlineformset_factory(
    Election, Category, extra=1,
    fields=['designation_pt', 'designation_en', 'weight'])


class ElectoralRollDocumentForm(DocumentForm):
    """ """

    def __init__(self, *args, **kwargs):
        """ """
        self.election = kwargs.pop('election')
        self.is_importing = kwargs.pop('is_importing')
        super(ElectoralRollDocumentForm, self).__init__(*args, **kwargs)
        if self.is_importing:
            for field in self.fields:
                self.fields[field].disabled = True

    def clean_document(self):
        """ """
        document = self.cleaned_data.get('document')
        importer = ElectoralRollImporter(self.election)
        errors = importer.validate(document, limit=5, verbose_errors=True)
        if errors:
            raise ValidationError(errors)
        return document


class CandidateForm(NestedFormsetFormMixin, forms.ModelForm):
    """ """
    formset_class = OrderedDocumentFormset
    formset_prefix = "documents"

    class Meta:
        model = Candidate
        fields = ('logo', 'designation')

    def get_queryset(self):
        """Return many to many queryset if candidate already exists on db"""
        if self.instance and self.instance.pk:
            return self.instance.documents.all()
        return Document.objects.none()

    def _save_m2m(self, *args, **kwargs):
        """Save and associates documents to the instance"""
        instances = self.formset.save(*args, **kwargs)
        self.instance.documents.add(*instances)
        super(CandidateForm, self)._save_m2m(*args, **kwargs)


CandidateFormset = inlineformset_factory(
    Election, Candidate, extra=0, min_num=1, form=CandidateForm)


class ElectionWriteInDocumentForm(DocumentForm):
    """ """

    class Meta(DocumentForm.Meta):
        fields = ('document',)

    def __init__(self, *args, **kwargs):
        """ """
        self.election = kwargs.pop('election')
        super(ElectionWriteInDocumentForm, self).__init__(*args, **kwargs)

    def clean_document(self):
        """ """
        document = self.cleaned_data.get('document')
        importer = CandidateImporter(self.election)
        errors = importer.validate(document, limit=5, verbose_errors=True)
        if errors:
            raise ValidationError(errors)
        return document


BulletinCandidateFormset = modelformset_factory(
    Candidate, extra=0, fields=("order",))


class ElectionBulletinForm(NestedFormsetFormMixin, forms.ModelForm):
    """ """
    formset_class = BulletinCandidateFormset
    formset_prefix = 'candidates'

    class Meta:
        model = Election
        fields = ("presentation_mode", "instructions_pt", "instructions_en")
        widgets = {
            "presentation_mode": forms.RadioSelect()
        }

    def get_queryset(self):
        """ """
        if self.instance and self.instance.pk:
            if self.instance.write_in:
                return self.instance.candidates.write_in()
            elif not self.instance.write_in:
                return self.instance.candidates.not_write_in()
        return Candidate.objects.none()

    def save(self, *args, **kwargs):
        """ """
        super(ElectionBulletinForm, self).save(*args, **kwargs)
        self.formset.save(*args, **kwargs)


class ElectoralMemberEmailForm(forms.Form):
    """ """

    email = forms.EmailField()

    class Meta:
        fields = ('email',)

    def __init__(self, *args, **kwargs):
        """ """
        self.process_pk = kwargs.pop('process_pk')
        super(ElectoralMemberEmailForm, self).__init__(*args, **kwargs)

    def clean_email(self):
        email = self.cleaned_data.get('email')
        self.set_user(email)
        return email

    def set_user(self, email):
        """ get the user from DTSI API """
        self.user = members_service.get_contact_info_by_email(email)
        if not self.user:
            raise ValidationError(_('The user does not exist'))

    def get_user(self):
        if hasattr(self, 'user'):
            return self.user
        return None


class ElectoralMemberForm(forms.ModelForm):
    """ """
    email = forms.EmailField()
    name = forms.CharField()

    class Meta:
        model = ElectoralMember
        fields = ('email', 'name', 'role', 'electoral_process', 'designation',
                  'phone')

    def __init__(self, *args, **kwargs):
        super(ElectoralMemberForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.initial['name'] = self.instance.user.name
            self.initial['email'] = self.instance.user.email
            self.fields['email'].widget.attrs['readonly'] = True
        else:
            self.fields['role'].widget.attrs['disabled'] = True
            self.fields['designation'].widget.attrs['readonly'] = True

        self.fields['name'].widget.attrs['readonly'] = True
        self.fields['phone'].widget.attrs['readonly'] = True

    def save(self, commit=True):
        """ """
        member, created = ElectoralMember.objects.update_or_create_member(
            email=self.cleaned_data.get('email'),
            name=self.cleaned_data.get('name'),
            electoral_process=self.cleaned_data.get('electoral_process'),
            designation=self.cleaned_data.get('designation'),
            role=self.cleaned_data.get('role'),
            phone=self.cleaned_data.get('phone'))
        return member


class ElectionManageOpenForm(forms.ModelForm):
    """ """
    error_messages = {
        'voting_begin_date_invalid':
            _("Must be higher than current date.")
    }

    class Meta:
        model = Election
        fields = ('voting_begin_date', )
        widgets = {
            'voting_begin_date': widgets.DateTimeInput(),
        }

    def __init__(self, *args, **kwargs):
        """ """
        super(ElectionManageOpenForm, self).__init__(*args, **kwargs)
        if not self.instance.is_in_definitive_eletoral_roll:
            for field in self.fields.values():
                field.disabled = True

    def clean_voting_begin_date(self):
        """ """
        voting_begin_date = self.cleaned_data.get('voting_begin_date')
        if voting_begin_date <= timezone.now():
            raise ValidationError(
                self.error_messages['voting_begin_date_invalid'])
        return voting_begin_date


class ElectionManageCloseForm(forms.ModelForm):
    """ """
    error_messages = {
        'voting_end_date_invalid': _("Must be higher than current date."),
        'voting_end_date_higher':
            _("Must be higher than 'voting begin date'."),
    }

    class Meta:
        model = Election
        fields = ('voting_end_date', )
        widgets = {
            'voting_end_date': widgets.DateTimeInput(),
        }

    def __init__(self, *args, **kwargs):
        super(ElectionManageCloseForm, self).__init__(*args, **kwargs)
        if not self.instance.is_in_to_vote:
            for field in self.fields.values():
                field.disabled = True

    def clean_voting_end_date(self):
        """ """
        voting_end_date = self.cleaned_data.get('voting_end_date')
        if voting_end_date <= timezone.now():
            raise ValidationError(
                self.error_messages['voting_end_date_invalid'])
        elif voting_end_date <= self.instance.voting_begin_date:
            raise ValidationError(
                self.error_messages['voting_end_date_higher'])
        return voting_end_date


class ElectionManageScrutinizeForm(forms.ModelForm):
    """ """

    class Meta:
        model = Election
        fields = ('id', )

    def __init__(self, election=None, *args, **kwargs):
        super(ElectionManageScrutinizeForm, self).__init__(*args, **kwargs)

    def clean(self):
        """ """
        return {'id': self.instance.pk}


class MembersFormSet(BaseInlineFormSet):
    """ """

    def clean(self):
        super(MembersFormSet, self).clean()
        chairman = False
        users = []
        for form in self.forms:
            email = form.cleaned_data.get('email')
            if email not in users:
                users.append(email)
            else:
                form.add_error('email', _('User is a member already'))
            if form.cleaned_data.get('role') == ElectoralMember.Role.CHAIRMAN\
                    and not chairman:
                chairman = True
            elif form.cleaned_data.get('role') ==\
                    ElectoralMember.Role.CHAIRMAN:
                form.add_error('role', _('There can only be one chairman.'))


ElectoralMembersFormset = inlineformset_factory(
    ElectoralProcess, ElectoralMember, extra=0, min_num=0,
    form=ElectoralMemberForm, formset=MembersFormSet)


class CertificateGenerationPresForm(forms.Form):
    """
    This form is used for presentation in browser only
    """
    password = forms.CharField(widget=forms.PasswordInput())
    ca = forms.CharField(widget=forms.HiddenInput())

    def __init__(self, *args, **kwargs):
        self.process = kwargs.pop("process")
        super(CertificateGenerationPresForm, self).__init__(*args, **kwargs)
        self.members_fields = []
        self.elections_fields = []
        for member in self.process.electoral_members.all():
            self.fields["secret%d" % member.pk] = forms.CharField(
                widget=forms.HiddenInput(attrs={'data-member-id': member.pk}))
            self.members_fields.append("secret%d" % member.pk)

        for election in self.process.elections.all():
            self.fields["csr%d" % election.pk] = forms.CharField(
                widget=forms.HiddenInput(
                    attrs={'data-election-id': election.pk}))
            self.elections_fields.append("csr%d" % election.pk)

    def get_members_fields(self):
        """ """
        return [field for field in self
                if field.name in self.members_fields]

    def get_elections_fields(self):
        """ """
        return [field for field in self
                if field.name in self.elections_fields]


class CertificateGenerationForm(forms.Form):
    elections_cert = forms.CharField(widget=forms.HiddenInput())

    class Meta:
        fields = ('elections_cert',)
