from __future__ import unicode_literals

import json
from mock import MagicMock
from unittest import skip

from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.forms.models import model_to_dict
from django.core.files import File
from django.utils import timezone, formats
from django.conf import settings

from evotevotings.elections.tests.utils import ElectionsTestUtils
from evotevotings.management.tables import (ElectoralProcessInstitutionalTable,
                                            ElectoralProcessMemberTable)
from evotevotings.management.forms import (ManagementAuthenticationForm,
                                           ElectoralProcessAsInstitutionalForm,
                                           ElectoralProcessAsMemberForm)
from evotevotings.processes.models import ElectoralMember, ElectoralProcess
from evotevotings.elections.models import Election
from evotevotings.processes.utils import prepopulate_electoralactions
from evoteajaxforms.views import AjaxResponseAction
from evotemessages.models import Message
from evotemessages.registers import recipient_register
from evotemessages.tests.utils import MessagesUtils
from guardian.shortcuts import assign_perm

from . import ManagementTestCase


class LoginViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.member = self.create_electoral_member(**self.credentials)
        self.url = settings.LOGIN_URL

    def test_login_view(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "management/login.html")

    def test_login_member(self):
        """ """
        self.credentials['profile'] = \
            ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER
        response = self.client.post(self.url, follow=True,
                                    data=self.credentials)
        self.assertRedirects(response, settings.LOGIN_REDIRECT_URL)
        self.assertEqual(response.context[0]["user"], self.member.user)

    def test_login_member_with_wrong_credentials(self):
        self.credentials["password"] = "wrong_password"
        self.credentials['profile'] = \
            ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER
        response = self.client.post(self.url, data=self.credentials)
        error_message = \
            ManagementAuthenticationForm.error_messages['invalid_login']
        self.assertFormError(response, "form", None, error_message)


class LogoutViewTest(ManagementTestCase):

    def setUp(self):
        self.login_as_member()

    def test_logout_view(self):
        """ """
        response = self.client.get(reverse("management:logout"), follow=True)
        self.assertRedirects(response, settings.LOGIN_URL)
        self.assertIsInstance(response.context[0]["user"], AnonymousUser)


class ElectoralProcessListViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.url = reverse("management:electoralprocess-list")

    def test_used_template(self):
        """ """
        self.login_as_member()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_list.html")

    def test_used_table_as_institutional(self):
        self.login_as_institutional()
        response = self.client.get(self.url)
        table_used = isinstance(
            response.context['table'], ElectoralProcessInstitutionalTable)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(table_used, True)

    def test_used_table_as_member(self):
        self.login_as_member()
        response = self.client.get(self.url)
        table_used = isinstance(
            response.context['table'], ElectoralProcessMemberTable)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(table_used, True)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class DetailViewTest(ManagementTestCase):

    def setUp(self):
        self.login_as_member()
        self.url = reverse("management:electoralprocess-detail", kwargs={
            "slug": self.member.electoral_process.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["object"].pk,
                         self.member.electoral_process.pk)
        self.assertTemplateUsed(
            response, "management/electoralprocess_detail.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectoralActionViewTest(ManagementTestCase):

    fixture = ['actions.json']

    def setUp(self):
        self.login_as_member()
        self.url = reverse("management:electoralprocess-actions", kwargs={
            "slug": self.member.electoral_process.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["object"].pk,
                         self.member.electoral_process.pk)
        self.assertTemplateUsed(
            response, "management/electoralprocess_actions.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_post_with_action_validated(self):
        data = {
            'electoral_actions-TOTAL_FORMS': '2',
            'electoral_actions-INITIAL_FORMS': '2',
            'electoral_actions-0-action': '1',
            'electoral_actions-0-id': '1',
            'electoral_actions-0-strength': '2',
            'electoral_actions-1-action': '2',
            'electoral_actions-1-id': '2',
            'electoral_actions-1-strength': '1',
        }
        # register manage action as validated
        self.register_action(self.member.electoral_process.pk, "manage")
        prepopulate_electoralactions(self.member.electoral_process)
        response = self.client.post(self.url, data=data)
        electoral_actions = self.member.electoral_process\
            .electoral_actions.all()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(electoral_actions[0].strength, 2)
        self.assertEqual(electoral_actions[1].strength, 1)

    def test_post_with_action_not_validated(self):
        data = {
            'electoral_actions-TOTAL_FORMS': '1',
            'electoral_actions-INITIAL_FORMS': '1',
            'electoral_actions-0-action': '1',
            'electoral_actions-0-id': '1',
            'electoral_actions-0-strength': '2',
        }
        prepopulate_electoralactions(self.member.electoral_process)
        response = self.client.post(self.url, data=data)
        self.assertEqual(response.status_code, 200)
        content = json.loads(response.content)
        self.assertIn('action', content)
        self.assertIn(content['action'], "refresh")


class ElectionListViewTest(ElectionsTestUtils, ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election(
            electoral_process=self.member.electoral_process)
        self.url = reverse("management:election-list", kwargs={
            "slug": self.member.electoral_process.slug
        })

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "management/election_list.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_queryset_filtered_by_electoral_process(self):
        """ """
        response = self.client.get(self.url)
        qs = response.context[0]['object_list']
        self.assertIn(self.election, list(qs))

    def test_view_extra_context_data(self):
        """ """
        response = self.client.get(self.url)
        self.assertIn('electoral_process', response.context[0])
        self.assertEqual(self.election.electoral_process_id,
                         response.context[0]['electoral_process'].pk)


class ElectoralProcessFormViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process(
            created_by=self.member.user)
        self.url = reverse("management:electoralprocess-new")
        self.edit_url = reverse("management:electoralprocess-edit", kwargs={
            "slug": self.process.slug
        })

    def test_used_template_in_create_view(self):
        """ """
        assign_perm("evoteprocesses.add_electoralprocess", self.member.user)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_form.html")

    def test_used_template_in_edit_view(self):
        """ """
        assign_perm("evoteprocesses.change_electoralprocess", self.member.user,
                    self.process)
        response = self.client.get(self.edit_url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_form.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_used_form_in_create_view_as_institutional(self):
        """ """
        assign_perm("evoteprocesses.add_electoralprocess", self.member.user)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.context['form'],
                              ElectoralProcessAsInstitutionalForm)

    def test_used_form_in_edit_view_as_institutional(self):
        """ """
        assign_perm("evoteprocesses.change_electoralprocess", self.member.user,
                    self.process)
        response = self.client.get(self.edit_url)
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.context['form'],
                              ElectoralProcessAsInstitutionalForm)

    def test_used_form_as_member(self):
        """ """
        assign_perm("evoteprocesses.manage_electoralprocess", self.member.user,
                    self.process)
        self.set_as_member()
        response = self.client.get(self.edit_url)
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.context['form'],
                              ElectoralProcessAsMemberForm)

    def test_post_valid_as_responsible(self):
        """ """
        assign_perm("evoteprocesses.add_electoralprocess", self.member.user)
        data = {
            'process-identifier_pt': 'teste',
            'process-identifier_en': 'test',
            'process-description_pt': 'description in pt',
            'process-description_en': 'description in en',
        }
        response = self.client.post(self.url, data=data)
        self.assertEqual(response.status_code, 200)
        content = json.loads(response.content)
        self.assertNotIn('errors_list', content)

    def test_post_valid_as_member(self):
        """ """
        assign_perm("evoteprocesses.manage_electoralprocess", self.member.user,
                    self.process)
        file_mock = MagicMock(spec=File, name='FileMock')
        file_mock.name = 'test1.jpg'
        date = formats.date_format(
            timezone.now() + timezone.timedelta(days=1),
            "DATETIME_FORMAT")
        data = {
            'process-identifier_pt': 'teste',
            'process-identifier_en': 'test',
            'process-description_pt': 'description in pt',
            'process-description_en': 'description in en',
            'process-publication_date': date,
            'electoral-documents-TOTAL_FORMS': 1,
            'electoral-documents-INITIAL_FORMS': 0,
            'electoral-documents-MIN_NUM_FORMS': 1,
            'electoral-documents-MAX_NUM_FORMS': 1000,
            'electoral-documents-0-designation': 'teste',
            'electoral-documents-0-documents-TOTAL_FORMS': 1,
            'electoral-documents-0-documents-INITIAL_FORMS': 0,
            'electoral-documents-0-documents-MIN_NUM_FORMS': 0,
            'electoral-documents-0-documents-MAX_NUM_FORMS': 1000,
            'electoral-documents-0-id': '',
            'electoral-documents-0-documents-0-designation': 'test',
            'electoral-documents-0-documents-0-id': '',
            'electoral-documents-0-documents-0-document': file_mock,

        }
        self.set_as_member()
        response = self.client.post(self.edit_url, data=data)
        self.assertEqual(response.status_code, 200)
        content = json.loads(response.content)
        self.assertNotIn('errors_list', content)

    def test_post_invalid_as_responsible(self):
        """ """
        assign_perm("evoteprocesses.add_electoralprocess", self.member.user)
        data = {
            'id_process-identifier_pt': 'teste',
            'id_process-identifier_en': 'test',
            'id_process-description_pt': 'description in pt',
            'id_process-description_en': 'description in en',
        }
        response = self.client.post(self.url, data=data)
        self.assertEqual(response.status_code, 200)
        content = json.loads(response.content)
        self.assertIn('errors_list', content)

    def test_post_invalid_as_member(self):
        """ """
        assign_perm("evoteprocesses.manage_electoralprocess", self.member.user,
                    self.process)
        date = formats.date_format(
            timezone.now() + timezone.timedelta(days=1),
            "DATETIME_FORMAT")
        data = {
            'process-identifier_pt': 'teste',
            'process-identifier_en': 'test',
            'process-description_pt': 'description in pt',
            'process-description_en': 'description in en',
            'process-publication_date': date,
            'electoral-documents-TOTAL_FORMS': 1,
            'electoral-documents-INITIAL_FORMS': 1,
            'electoral-documents-MIN_NUM_FORMS': 1,
            'electoral-documents-MAX_NUM_FORMS': 1000,
            'electoral-documents-0-designation': '',
            'electoral-documents-0-documents-TOTAL_FORMS': 2,
            'electoral-documents-0-documents-INITIAL_FORMS': 1,
            'electoral-documents-0-documents-MIN_NUM_FORMS': 0,
            'electoral-documents-0-documents-MAX_NUM_FORMS': 1000,
            'electoral-documents-0-id': 1,
            'electoral-documents-0-documents-0-designation': 'test',
            'electoral-documents-0-documents-0-id': 1,
        }
        self.set_as_member()
        response = self.client.post(self.edit_url, data=data)
        self.assertEqual(response.status_code, 200)
        content = json.loads(response.content)
        self.assertIn('errors_list', content)


class ElectoralProcessElectionsViewTest(ElectionsTestUtils,
                                        ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process(
            created_by=self.member.user)
        self.election = self.create_election(electoral_process=self.process)
        self.url = reverse("management:electoralprocess-elections", kwargs={
            "slug": self.process.slug
        })
        self.data_valid = {
            'elections-TOTAL_FORMS': '1',
            'elections-INITIAL_FORMS': '0',
            'elections-MIN_NUM_FORMS': '1',
            'elections-MAX_NUM_FORMS': '1000',
            'elections-0-identifier_pt': 'election test in pt',
            'elections-0-identifier_en': 'election test in en',
            'elections-0-description_pt': 'description election in pt',
            'elections-0-description_en': 'description election in en',
            'elections-0-proposition_begin_date': '2016-10-26 15:00:00',
            'elections-0-proposition_end_date': '2016-10-27 15:00:00',
            'elections-0-roll_begin_date': '2016-10-28 15:00:00',
            'elections-0-roll_end_date': '2016-10-29 15:00:00',
            'elections-0-voting_begin_date': '2016-10-30 15:00:00',
            'elections-0-voting_end_date': '2016-11-01 15:00:00',
            'elections-0-voting_references_date': '2016-11-02 15:00:00',
            'elections-0-minute_publication_date': '2016-11-03 15:00:00',
            'elections-0-minute_begin_date': '2016-11-04 15:00:00',
            'elections-0-minute_end_date': '2016-11-05 15:00:00',
        }

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_elections.html")

    def test_post_valid(self):
        """ """
        response = self.client.post(self.url, data=self.data_valid)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    def test_post_invalid(self):
        """ """
        data = {
            'elections-TOTAL_FORMS': '1',
            'elections-INITIAL_FORMS': '0',
            'elections-MIN_NUM_FORMS': '1',
            'elections-MAX_NUM_FORMS': '1000',
            'elections-0-identifier_pt': 'election test in pt',
            'elections-0-identifier_en': 'election test in en',
            'elections-0-description_pt': '',
            'elections-0-description_en': '',
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('errors_list', content)

    def test_post_valid_as_process_new(self):
        """ """
        self.process.state = ElectoralProcess.State.NEW
        self.process.save()
        response = self.client.post(self.url, data=self.data_valid)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('action', content)
        self.assertEqual(content['action'], "refresh")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


@skip('Need to check this')
class ElectoralProcessMembersEditViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        """ """
        credentials = {
            "email": "dev@eurotux.com",
            "password": "password",
            "role": 1
        }
        self.member = self.create_institutional_member(**credentials)
        self.client.login(**credentials)
        self.set_as_institutional()
        self.process = self.create_electoral_process(
            state=ElectoralProcess.State.NEW)
        self.url = reverse(
            "management:electoralprocess-members-edit", kwargs={
                "slug": self.process.slug
            })
        assign_perm("evoteprocesses.change_electoralprocess", self.member.user,
                    self.process)
        self.valid_data = {
            'members-TOTAL_FORMS': 1,
            'members-INITIAL_FORMS': 0,
            'members-MIN_NUM_FORMS': 0,
            'members-MAX_NUM_FORMS': 1000,
            'members-0-email': 'tfb@eurotux.com',
            'members-0-name': 'Tiago Brito',
            'members-0-role': '0',
            'members-0-designation': 'test',
            'members-0-phone': '+351912233445'
        }

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_members.html")

    def test_with_unexistent_object(self):
        """ """
        url = reverse("management:electoralprocess-members-edit", kwargs={
            "slug": "test"
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_form_valid(self):
        response = self.client.post(self.url, data=self.valid_data)
        created = ElectoralMember.objects.filter(
            user__email=self.valid_data['members-0-email']).exists()
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(created, True)
        action_url = reverse("management:electoralprocess-generate",
                             kwargs={"slug": self.process.slug})
        self.assertIn('redirect', content['action'])
        self.assertIn(action_url, content['action_url'])

    def test_form_valid_as_draft(self):
        self.process.state = ElectoralProcess.State.DRAFT
        self.process.save()
        response = self.client.post(self.url, data=self.valid_data)
        created = ElectoralMember.objects.filter(
            user__email=self.valid_data['members-0-email']).exists()
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(created, True)
        action_url = reverse("management:electoralprocess-summary",
                             kwargs={"slug": self.process.slug})
        self.assertIn('redirect', content['action'])
        self.assertIn(action_url, content['action_url'])

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectoralProcessMembersViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.process = self.create_electoral_process(
            state=ElectoralProcess.State.NEW)
        self.url = reverse(
            "management:electoralprocess-members-view", kwargs={
                "slug": self.process.slug
            })

    @skip('Need to check this')
    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_members.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectoralProcessSummaryViewTest(ElectionsTestUtils, ManagementTestCase):
    """ """

    def setUp(self):
        # TODO: improve the setUp
        self.login_as_institutional()
        self.process = self.create_electoral_process(
            created_by=self.member.user)
        self.member.electoral_process = self.process
        self.member.role = ElectoralMember.Role.CHAIRMAN
        self.member.save()
        self.url = reverse("management:electoralprocess-summary", kwargs={
            "slug": self.process.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_summary.html")

    def test_redirect_when_no_chairman_associated(self):
        """ """
        self.member.role = ElectoralMember.Role.OTHER
        self.member.save()
        response = self.client.get(self.url)
        redirect_to = reverse("management:electoralprocess-members-edit",
                              kwargs={"slug": self.process.slug})
        self.assertRedirects(response, redirect_to)

    def test_redirect_when_no_members_associated(self):
        """ """
        self.member.delete()
        response = self.client.get(self.url)
        redirect_to = reverse("management:electoralprocess-members-edit",
                              kwargs={"slug": self.process.slug})
        self.assertRedirects(response, redirect_to)

    def test_with_unexistent_object(self):
        """ """
        url = reverse("management:electoralprocess-summary", kwargs={
            "slug": "test"
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)


class ElectoralProcessConfirmationViewTest(ManagementTestCase,
                                           ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process(state=4)
        self.election = self.create_election(electoral_process=self.process)
        self.member = self.create_electoral_member(
            electoral_process=self.process, role=ElectoralMember.Role.CHAIRMAN)
        self.url = reverse(
            "management:electoralprocess-confirmation", kwargs={
                "slug": self.process.slug
            })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_confirmation.html")

    def test_cannot_proceed(self):
        """ """
        process = self.create_electoral_process(electoral_members=None)
        url = reverse(
            "management:electoralprocess-confirmation", kwargs={
                "slug": process.slug
            })
        response = self.client.get(url)
        message = list(response.context['messages'])[0]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(str(message), "Unable to submit.")

    def test_with_unexistent_object(self):
        """ """
        url = reverse("management:electoralprocess-confirmation", kwargs={
            "slug": "test"
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)


class ElectoralProcessCommunicationViewTest(ManagementTestCase,
                                            ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_member()
        self.process = self.create_electoral_process(state=4)
        self.election = self.create_election(electoral_process=self.process)
        self.member = self.create_electoral_member(
            electoral_process=self.process, role=ElectoralMember.Role.CHAIRMAN,
            is_comunication_responsible=True)
        self.url = reverse(
            "management:electoralprocess-communication", kwargs={
                "slug": self.process.slug
            })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_communication.html")

    def test_change_responsible(self):
        data = {
            'responsible-responsible_communication': self.member.pk
        }
        response = self.client.post(self.url, data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    def test_with_unexistent_object(self):
        """ """
        url = reverse("management:electoralprocess-communication", kwargs={
            "slug": "test"
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)


class ElectionDetailConfigsViewTest(ManagementTestCase, ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election(
            electoral_process=self.member.electoral_process)
        self.url = reverse("management:election-detail-configs", kwargs={
            "slug": self.member.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_get(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_configs.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_success_url(self):
        data = model_to_dict(self.election)
        # garantee that election has hondt_total_seats correctly filled
        data.update({
            'hondt_total_seats': 1
        })
        response = self.client.post(self.url, data)
        content = json.loads(response.content)
        success_url = reverse("management:election-detail-calendar", kwargs={
            'slug': self.election.electoral_process.slug,
            'slug_election': self.election.slug
        })
        self.assertEqual(response.status_code, 200)
        self.assertEqual(content['action'], AjaxResponseAction.REDIRECT)
        self.assertEqual(content['action_url'], success_url)


class ElectionDetailCalendarViewTest(ManagementTestCase, ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election(
            electoral_process=self.member.electoral_process)
        self.url = reverse("management:election-detail-calendar", kwargs={
            "slug": self.member.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_get(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_calendar.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_success_url(self):
        data = {
            'proposition_begin_date': '10-10-2016 10:00',
            'proposition_end_date': '11-10-2016 10:00',
            'roll_begin_date': '12-10-2016 10:00',
            'roll_end_date': '13-10-2016 10:00',
            'voting_begin_date': '14-10-2016 10:00',
            'voting_end_date': '15-10-2016 10:00',
            'voting_references_date': '15-10-2016 10:00',
            'minute_publication_date': '16-10-2016 10:00',
            'minute_begin_date': '17-10-2016 10:00',
            'minute_end_date': '18-10-2016 10:00'
        }
        response = self.client.post(self.url, data)
        content = json.loads(response.content)
        success_url = reverse("management:election-detail-categories", kwargs={
            'slug': self.election.electoral_process.slug,
            'slug_election': self.election.slug
        })
        self.assertEqual(response.status_code, 200)
        self.assertEqual(content['action'], AjaxResponseAction.REDIRECT)
        self.assertEqual(content['action_url'], success_url)


class ElectionDetailCategoriesViewTest(ManagementTestCase, ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election(
            electoral_process=self.member.electoral_process)
        self.url = reverse("management:election-detail-categories", kwargs={
            "slug": self.member.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_categories.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_success_url(self):
        data = {
            'categories-TOTAL_FORMS': '1',
            'categories-INITIAL_FORMS': '1',
            'categories-0-id': '1',
            'categories-0-election': self.election.pk,
            'categories-0-designation_pt': 'Um',
            'categories-0-designation_en': 'One',
            'categories-0-weight': '1'
        }
        response = self.client.post(self.url, data)
        content = json.loads(response.content)
        success_url = reverse("management:election-detail-rolls", kwargs={
            'slug': self.election.electoral_process.slug,
            'slug_election': self.election.slug
        })
        self.assertEqual(response.status_code, 200)
        self.assertEqual(content['action'], AjaxResponseAction.REDIRECT)
        self.assertEqual(content['action_url'], success_url)


class ElectionDetailCandidatesViewTest(ElectionsTestUtils, ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election(
            electoral_process=self.member.electoral_process)
        self.url = reverse("management:election-detail-candidates", kwargs={
            "slug": self.member.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_candidates.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionDetailBulletinViewTest(ElectionsTestUtils, ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election(
            electoral_process=self.member.electoral_process)
        self.url = reverse("management:election-detail-bulletin", kwargs={
            "slug": self.member.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_bulletin.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionDetailBulletinPreviewViewTest(ElectionsTestUtils,
                                            ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        election = self.create_election(
            electoral_process=self.member.electoral_process)
        assign_perm("evoteprocesses.vote_election", self.member.user,
                    election)
        self.url = reverse("management:election-detail-bulletin-preview",
                           kwargs={"slug": self.member.electoral_process.slug,
                                   "slug_election": election.slug})

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_bulletin_preview.html")


class PersonalDataViewTest(ManagementTestCase):

    def setUp(self):
        self.login_as_member()
        self.url = reverse("management:personal-data")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'management/personal_data.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionManageViewTest(ElectionsTestUtils, ManagementTestCase):

    fixture = ['actions.json']

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election(
            state=Election.State.DEFINITIVE_ELECTORAL_ROLL,
            electoral_process=self.member.electoral_process)
        self.url = reverse("management:election-manage", kwargs={
            "slug": self.member.electoral_process.slug,
            "slug_election": self.election.slug
        })
        self.register_action(self.member.electoral_process.pk, 1)

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["object"].pk, self.election.pk)
        self.assertTemplateUsed(
            response, "management/election_manage.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_post_with_election_in_definitive_eletoral_roll(self):
        """ """
        expected = formats.date_format(
            timezone.now() + timezone.timedelta(days=1), "DATETIME_FORMAT")
        data = {
            'voting_begin_date': expected
        }
        response = self.client.post(self.url, data)
        self.assertEqual(response.status_code, 200)
        self.election.refresh_from_db()
        returned = formats.date_format(
            self.election.voting_begin_date, "DATETIME_FORMAT")
        self.assertEqual(returned, expected)

    def test_post_with_election_not_in_definitive_eletoral_roll(self):
        """ """
        self.election.state = self.election.State.IN_TO_VOTE
        self.election.save()
        expected = formats.date_format(
            timezone.now() + timezone.timedelta(days=1), "DATETIME_FORMAT")
        data = {
            'voting_begin_date': expected
        }
        response = self.client.post(self.url, data)
        self.assertEqual(response.status_code, 200)
        self.election.refresh_from_db()
        returned = formats.date_format(
            self.election.voting_begin_date, "DATETIME_FORMAT")
        self.assertNotEqual(returned, expected)


class ElectionManageCloseViewTest(ElectionsTestUtils, ManagementTestCase):

    fixture = ['actions.json']

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election(
            state=Election.State.IN_TO_VOTE,
            electoral_process=self.member.electoral_process)
        self.url = reverse("management:election-manage-close", kwargs={
            "slug": self.member.electoral_process.slug,
            "slug_election": self.election.slug
        })
        self.register_action(self.member.electoral_process.pk, 2)

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["object"].pk, self.election.pk)
        self.assertTemplateUsed(
            response, "management/election_manage_close.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_post_with_election_in_to_vote(self):
        """ """
        expected = formats.date_format(
            timezone.now() + timezone.timedelta(days=1), "DATETIME_FORMAT")
        data = {
            'voting_end_date': expected
        }
        response = self.client.post(self.url, data)
        self.assertEqual(response.status_code, 200)
        self.election.refresh_from_db()
        returned = formats.date_format(
            self.election.voting_end_date, "DATETIME_FORMAT")
        self.assertEqual(returned, expected)

    def test_post_with_election_not_in_to_vote(self):
        """ """
        self.election.state = self.election.State.IN_COUNTING
        self.election.save()
        expected = formats.date_format(
            timezone.now() + timezone.timedelta(days=1), "DATETIME_FORMAT")
        data = {
            'voting_end_date': expected
        }
        response = self.client.post(self.url, data)
        self.assertEqual(response.status_code, 200)
        self.election.refresh_from_db()
        returned = formats.date_format(
            self.election.voting_end_date, "DATETIME_FORMAT")
        self.assertNotEqual(returned, expected)


class GenerateCertificateViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process(state=4)
        self.url = reverse("management:electoralprocess-generate", kwargs={
            "slug": self.process.slug,
        })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, 'management/generate_certificate.html')

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessagesRecievedListViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.url = reverse(
            "management:messages-recieved", kwargs={
                "slug": self.member.electoral_process.slug,
            })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, 'management/evotemessages/message_list.html')

    def test_used_queryset(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        expected = Message.objects.inbox(
            self.member.user, self.member.electoral_process)
        self.assertQuerysetEqual(
            response.context["object_list"], map(repr, expected))

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessagesSentListViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.url = reverse(
            "management:messages-sent", kwargs={
                "slug": self.member.electoral_process.slug,
            })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, 'management/evotemessages/message_list.html')

    def test_used_queryset(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        expected = Message.objects.sentbox(
            self.member.user, self.member.electoral_process)
        self.assertQuerysetEqual(
            response.context["object_list"], map(repr, expected))

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessagesArchivedListViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.url = reverse(
            "management:messages-archived", kwargs={
                "slug": self.member.electoral_process.slug,
            })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, 'management/evotemessages/message_list.html')

    def test_used_queryset(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        expected = Message.objects.archivebox(
            self.member.user, self.member.electoral_process)
        self.assertQuerysetEqual(
            response.context["object_list"], map(repr, expected))

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessageNewManageViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        setattr(self.member.user, 'is_electoral_member', True)
        self.url = reverse(
            "management:message-new", kwargs={
                "slug": self.member.electoral_process.slug,
            })
        self.success_url = reverse(
            "management:messages-recieved", kwargs={
                "slug": self.member.electoral_process.slug,
            })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, 'management/evotemessages/message_form.html')

    def test_success_url(self):
        self.recipient = recipient_register.get_all_recipients(
            self.member.user, q='',
            slug=[self.member.electoral_process.slug])[0]\
            .get('id')
        data = {
            'newmsg-name': self.member.user.name,
            'newmsg-email': self.member.user.email,
            'newmsg-recipients': [self.recipient],
            'newmsg-subject': 'Subject of message',
            'newmsg-body': 'body of message',
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(content['action'], AjaxResponseAction.REDIRECT)
        self.assertEqual(content['action_url'], self.success_url)

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class MessageResponseManageViewTest(MessagesUtils, ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.message = self.create_message(sender=self.member.user)
        setattr(self.member.user, 'is_electoral_member', True)
        assign_perm("view_message", self.member.user, self.message)
        self.url = reverse(
            "management:message-response", kwargs={
                "msg_slug": self.message.slug,
                "slug": self.member.electoral_process.slug,
            })
        self.success_url = reverse(
            "management:messages-recieved", kwargs={
                "slug": self.member.electoral_process.slug,
            })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, 'management/evotemessages/message_form.html')

    def test_success_url(self):
        data = {
            'remsg-name': self.member.user.name,
            'remsg-email': self.member.user.email,
            'remsg-subject': 'Subject of message',
            'remsg-body': 'body of message',
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(content['action'], AjaxResponseAction.REDIRECT)
        self.assertEqual(content['action_url'], self.success_url)

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)
