from __future__ import unicode_literals

from django.contrib.auth.decorators import login_required
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.base import View
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy
from django.utils.decorators import method_decorator
from django.views.generic import ListView, DetailView
from django.shortcuts import get_object_or_404, redirect
from django.contrib import messages
from django.utils import timezone

from evotevotings.elections.importers import CandidateImporter
from evotevotings.elections.models import Election, ElectoralRoll
from evotevotings.processes.models import ElectoralProcess, ElectoralMember
from evotevotings.processes.utils import prepopulate_electoralactions
from evotevotings.elections.forms import ElectoralCalendarForm
from evotevotings.elections.tasks import (import_electoral_roll,
                                          open_election_voting,
                                          close_election_voting)
from evotevotings.voters.views import ElectionVotingStep1View
from evoteajaxforms.views import (FormJsonView, FormsetJsonView,
                                  AjaxResponseAction, FormJsonSimpleView)
from evoteusers.views import EvoteLoginView, PersonalDataBaseView
from evotecore.mixins import PermissionListMixin, PermissionRequiredMixin
from evotemessages.views import (MessagesRecievedView, MessagesSentView,
                                 MessagesArchivedView, MessageNewView,
                                 MessageView, ArchiveMessageView,
                                 DeleteMessageView, MessageResponseView)
from evotemessages.models import Message
from evotemessages.templatetags.messages_tags import nested_url
from evotemessages.registers import recipient_register
from django_tables2 import SingleTableMixin
from django_fsm import can_proceed
from guardian.shortcuts import assign_perm

from .middleware import SESSION_USER_PROFILE
from .mailings import MembersKeysNotificationMail
from .mixins import ActionValidationMixin
from .tables import (ElectoralProcessMemberTable, ElectoralProcessDetailTable,
                     ElectoralProcessInstitutionalTable, ElectionTable,
                     ElectoralMemberSimpleTable, ElectoralRollTable,
                     ElectionVoterTable, ElectionVoteCountingTable,
                     ElectionVoteCountingExtraTable)
from .forms import (ManagementAuthenticationForm, ElectoralActionFormset,
                    ElectionDetailConfigsForm, ElectionFormset,
                    CategoryFormset, ElectoralRollDocumentForm,
                    ElectoralProcessAsMemberForm,
                    ElectoralProcessCommunicationForm,
                    ElectoralProcessAsInstitutionalForm,
                    ElectoralDocumentFormset, CandidateFormset,
                    ElectionWriteInDocumentForm, ElectionBulletinForm,
                    ElectoralMembersFormset, CertificateGenerationForm,
                    CertificateGenerationPresForm,
                    ElectionManageOpenForm, ElectionManageCloseForm,
                    ElectionManageScrutinizeForm)

from .mixins import MessageViewManageMixin, MessageMixin


class LoginView(EvoteLoginView):
    """Provides user the ability to login"""
    form_class = ManagementAuthenticationForm
    template_name = "management/login.html"

    def form_valid(self, form):
        self.request.session[SESSION_USER_PROFILE] = \
            form.cleaned_data.get('profile')
        return super(LoginView, self).form_valid(form)


"""
View electoral process's
"""


@method_decorator(login_required, name="dispatch")
class ElectoralProcessListView(PermissionListMixin, SingleTableMixin,
                               ListView):
    """ """
    model = ElectoralProcess
    queryset = ElectoralProcess.objects.public()
    permission_required = "evoteprocesses.manage_electoralprocess"
    template_name = "management/electoralprocess_list.html"

    table_class = ElectoralProcessMemberTable
    table_pagination = {'per_page': 5}

    def get_required_permissions(self, request=None):
        """ """
        if self.request.user.is_institutional_responsible:
            return ['evoteprocesses.change_electoralprocess']
        return super(ElectoralProcessListView, self)\
            .get_required_permissions(request)

    def get_queryset(self, **kwargs):
        """ """
        if self.request.user.is_institutional_responsible:
            return ElectoralProcess.objects.filter(
                created_by=self.request.user)
        return super(ElectoralProcessListView, self).get_queryset(**kwargs)

    def get_table_class(self):
        """ """
        if self.request.user.is_institutional_responsible:
            return ElectoralProcessInstitutionalTable
        return super(ElectoralProcessListView, self).get_table_class()


@method_decorator(login_required, name="dispatch")
class ElectoralProcessDetailView(PermissionRequiredMixin, SingleTableMixin,
                                 DetailView):
    """ """
    model = ElectoralProcess
    template_name = "management/electoralprocess_detail.html"
    permission_required = "evoteprocesses.manage_electoralprocess"

    table_class = ElectoralProcessDetailTable
    table_pagination = False

    def get_table_data(self):
        return [self.object]


@method_decorator(login_required, name="dispatch")
class ElectoralProcessActionsView(PermissionRequiredMixin,
                                  ActionValidationMixin, FormJsonView):
    """ """
    model = ElectoralProcess
    form_class = ElectoralActionFormset
    permission_required = "evoteprocesses.manage_electoralprocess"
    action_validation_pk = "manage"
    template_name = "management/electoralprocess_actions.html"
    action = AjaxResponseAction.REFRESH

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        prepopulate_electoralactions(self.object)
        return super(ElectoralProcessActionsView, self)\
            .get(request, *args, **kwargs)

    def form_valid(self, form):
        """ """
        if not self.object.actions_configured:
            self.object.actions_configured = True
            self.object.save(update_fields=["actions_configured"])
        messages.success(self.request, _("Operations updated successfully"))
        return super(ElectoralProcessActionsView, self).form_valid(form)


@method_decorator(login_required, name="dispatch")
class ElectoralProcessFormView(PermissionRequiredMixin, FormJsonView):
    """ """
    model = ElectoralProcess
    accept_global_perms = True
    template_name = "management/electoralprocess_form.html"
    prefix = 'process'
    formset = ElectoralDocumentFormset
    formset_prefix = 'electoral-documents'

    def get_required_permissions(self, request=None):
        """ """
        if not self.kwargs.get(self.slug_url_kwarg, None):
            return ['evoteprocesses.add_electoralprocess']
        elif self.request.user.is_institutional_responsible:
            return ['evoteprocesses.change_electoralprocess']
        return ['evoteprocesses.manage_electoralprocess']

    def get_form_class(self):
        """ """
        if self.request.user.is_institutional_responsible:
            return ElectoralProcessAsInstitutionalForm
        return ElectoralProcessAsMemberForm

    def form_valid(self, form):
        """ """
        if self.request.user.is_institutional_responsible:
            return self.handle_responsible_form(form)
        return self.handle_member_form(form)

    def handle_member_form(self, form):
        """ """
        documents_formset = self.formset(
            self.request.POST, self.request.FILES, prefix=self.formset_prefix,
            instance=self.object)
        if documents_formset.is_valid():
            form.save()
            documents_formset.save()
            messages.success(
                self.request, _("Operations executed successfully."))
            return self.json_to_response(action=AjaxResponseAction.REFRESH)
        return self.formset_invalid(documents_formset, self.formset_prefix)

    def handle_responsible_form(self, form):
        process = form.save(commit=False)
        if not self.object:
            process.created_by = self.request.user
        process.save()
        messages.success(
            self.request, _("Operations executed successfully."))
        if not self.object or process.is_draft():
            success_url = reverse_lazy("management:electoralprocess-elections",
                                       args=[process.slug])
            return self.json_to_response(success_url=success_url)
        return self.json_to_response(action=AjaxResponseAction.REFRESH)

    def get_context_data(self, **kwargs):
        """ """
        context = super(ElectoralProcessFormView, self)\
            .get_context_data(**kwargs)
        if self.request.user.is_electoral_member:
            context.update({
                'docformset': self.formset(
                    prefix=self.formset_prefix, instance=self.object)
            })
        return context

    def get_object(self):
        if self.kwargs.get(self.slug_url_kwarg, None):
            return super(ElectoralProcessFormView, self).get_object()
        return None


@method_decorator(login_required, name="dispatch")
class ElectoralProcessElectionsView(PermissionRequiredMixin, FormsetJsonView):
    """ """
    model = ElectoralProcess
    permission_required = 'evoteprocesses.change_electoralprocess'
    template_name = "management/electoralprocess_elections.html"
    form_class = ElectionFormset
    prefix = 'elections'

    def form_valid(self, form):
        if not self.object.is_draft():
            self.action = AjaxResponseAction.REFRESH
        messages.success(
            self.request, _("Operations executed successfully."))
        return super(ElectoralProcessElectionsView, self).form_valid(form)

    def get_success_url(self):
        """ """
        return reverse_lazy("management:electoralprocess-members-edit",
                            kwargs={
                                'slug': self.kwargs.get(self.slug_url_kwarg)
                            })


@method_decorator(login_required, name="dispatch")
class ElectoralProcessMembersView(PermissionRequiredMixin, SingleTableMixin,
                                  DetailView):
    """ """
    model = ElectoralProcess
    permission_required = 'evoteprocesses.manage_electoralprocess'
    template_name = "management/electoralprocess_members.html"
    table_class = ElectoralMemberSimpleTable

    def get_required_permissions(self, request=None):
        """ """
        if self.request.user.is_institutional_responsible:
            return ['evoteprocesses.change_electoralprocess']
        return super(ElectoralProcessMembersView, self)\
            .get_required_permissions(request)

    def get_table_data(self):
        return self.object.electoral_members.all()


@method_decorator(login_required, name="dispatch")
class ElectoralProcessMembersEditView(FormsetJsonView):
    model = ElectoralProcess
    template_name = "management/electoralprocess_members.html"
    form_class = ElectoralMembersFormset
    prefix = 'members'

    def get_success_url(self):
        # self.get_object is needed since there is a bug in ajaxforms
        if not self.get_object().is_draft():
            return reverse_lazy(
                "management:electoralprocess-generate", kwargs={
                    'slug': self.kwargs.get('slug')
                })
        return reverse_lazy(
            "management:electoralprocess-summary", kwargs={
                'slug': self.kwargs.get('slug')
            })

    def get_form_kwargs(self):
        form_kwargs = super(
            ElectoralProcessMembersEditView, self).get_form_kwargs()
        form_kwargs.update(
            {'form_kwargs': {'initial': {
                'electoral_process_id': self.object.pk}}}
        )
        return form_kwargs

    def form_valid(self, form):
        messages.success(
            self.request, _("Electoral comission updated."))
        return super(ElectoralProcessMembersEditView, self).form_valid(form)


@method_decorator(login_required, name="dispatch")
class ElectoralProcessSummaryView(
        PermissionRequiredMixin, SingleTableMixin, DetailView):
    """ """
    model = ElectoralProcess
    permission_required = 'evoteprocesses.change_electoralprocess'
    template_name = "management/electoralprocess_summary.html"

    table_class = ElectoralMemberSimpleTable
    table_pagination = False

    def get_table_data(self):
        return self.object.electoral_members.all()

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not self.object.electoral_members.all().filter(
                role=ElectoralMember.Role.CHAIRMAN).exists():
            messages.error(
                request, _("No chairman inserted! Please insert one."))
            return redirect('management:electoralprocess-members-edit',
                            slug=self.kwargs['slug'])
        return super(ElectoralProcessSummaryView, self)\
            .dispatch(request, *args, **kwargs)


@method_decorator(login_required, name="dispatch")
class ElectoralProcessConfirmationView(DetailView):
    """ """
    model = ElectoralProcess
    template_name = "management/electoralprocess_confirmation.html"

    def get(self, request, *args, **kwargs):
        response = super(ElectoralProcessConfirmationView, self).get(
            request, *args, **kwargs)
        if not can_proceed(self.object.submit_new):
            messages.error(request, _("Unable to submit."))
            return response
        # TODO: pass this code into submit_new transition method
        chairmans = ElectoralMember.objects.filter(
            electoral_process=self.object, role=ElectoralMember.Role.CHAIRMAN)
        if chairmans.exists():
            ElectoralMember.objects.mark_as_responsible(
                self.object, chairmans.first())
        self.object.submit_new()
        self.object.save()
        return response


@method_decorator(login_required, name="dispatch")
class ElectoralProcessCommunicationView(FormJsonView):
    """ """
    model = ElectoralProcess
    template_name = "management/electoralprocess_communication.html"
    form_class = ElectoralProcessCommunicationForm
    prefix = 'responsible'
    action = AjaxResponseAction.REFRESH

    def get_form_kwargs(self):
        form_kwargs = super(ElectoralProcessCommunicationView, self)\
            .get_form_kwargs()
        chairmans = self.object.electoral_members.filter(
            is_comunication_responsible=True)
        if chairmans.exists():
            form_kwargs.update({
                'initial': {
                    'responsible_communication': chairmans.first()
                }
            })
        return form_kwargs

    def form_valid(self, form):
        member = form.cleaned_data['responsible_communication']
        ElectoralMember.objects.mark_as_responsible(self.object, member)
        messages.success(
            self.request, _("Responsible of communication updated."))
        return super(ElectoralProcessCommunicationView, self).form_valid(
            form)


"""
Views elections
"""


class ElectionManagePermissionMixin(PermissionRequiredMixin):
    """ """
    permission_required = 'evoteprocesses.manage_electoralprocess'

    def get_electoral_process(self):
        """ """
        if not hasattr(self, 'electoral_process'):
            # defined here to be used on context_data and avoid more queries
            self.electoral_process = get_object_or_404(
                ElectoralProcess, slug=self.kwargs.get('slug'))
        return self.electoral_process

    def get_permission_object(self):
        return self.get_electoral_process()


@method_decorator(login_required, name="dispatch")
class ElectionListView(ElectionManagePermissionMixin, SingleTableMixin,
                       ListView):
    """ """
    model = Election
    table_class = ElectionTable
    template_name = "management/election_list.html"

    def get_queryset(self):
        return self.get_electoral_process().elections.all()

    def get_context_data(self, **kwargs):
        context = super(ElectionListView, self).get_context_data(**kwargs)
        context.update({
            "electoral_process": self.get_electoral_process()
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectionManageView(ElectionManagePermissionMixin, ActionValidationMixin,
                         FormJsonView):
    """ """
    model = Election
    form_class = ElectionManageOpenForm
    slug_url_kwarg = "slug_election"
    action_validation_pk = 1
    template_name = "management/election_manage.html"
    action = AjaxResponseAction.REFRESH

    def get_action_validation_key(self):
        return self.get_electoral_process().pk

    def form_valid(self, form):
        if not self.object.is_in_definitive_eletoral_roll:
            return self.json_to_response()
        response = super(ElectionManageView, self).form_valid(form)
        open_election_voting.si(self.object.id).apply_async(
            eta=self.object.voting_begin_date)
        return response


@method_decorator(login_required, name="dispatch")
class ElectionManageCloseView(ElectionManagePermissionMixin,
                              ActionValidationMixin, FormJsonView):
    """ """
    model = Election
    form_class = ElectionManageCloseForm
    slug_url_kwarg = "slug_election"
    action_validation_pk = 2
    template_name = "management/election_manage_close.html"
    action = AjaxResponseAction.REFRESH

    def get_action_validation_key(self):
        return self.get_electoral_process().pk

    def form_valid(self, form):
        if not self.object.is_in_to_vote:
            return self.json_to_response()
        response = super(ElectionManageCloseView, self).form_valid(form)
        close_election_voting.si(self.object.id).apply_async(
            eta=self.object.voting_end_date)
        return response


@method_decorator(login_required, name="dispatch")
class ElectionManageScrutinizeView(ElectionManagePermissionMixin,
                                   ActionValidationMixin, FormJsonView):
    """ """
    model = Election
    form_class = ElectionManageScrutinizeForm
    slug_url_kwarg = "slug_election"
    action_validation_pk = 3
    template_name = "management/election_manage_scrutinize.html"
    action = AjaxResponseAction.REFRESH

    def get_context_data(self, **kwargs):
        context = super(ElectionManageScrutinizeView, self)\
            .get_context_data(**kwargs)
        # TODO: make request to check if is already in couting or not and the
        # total votes
        context.update({
            'scrutinizing': False,
            'finished': False,
            'total_votes': 20000
        })
        if context['finished']:
            context.update({
                'results_table': ElectionVoteCountingTable({}),
                'results_extra_table': ElectionVoteCountingExtraTable([{
                    'invalid': 10, 'votes_submitted': 1000,
                    'votes_submitting': 0, 'votes_unsubmitted': 100
                }])
            })
        return context

    def form_valid(self, form):
        """ """
        # TODO: make request to counter service to start the scrutiny
        form.cleaned_data.get('id')
        return super(ElectionManageScrutinizeView, self).form_valid(form)


@method_decorator(login_required, name="dispatch")
class ElectionDetailConfigsView(ElectionManagePermissionMixin, FormJsonView):
    """ """
    model = Election
    form_class = ElectionDetailConfigsForm
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_configs.html"

    def form_valid(self, form):
        self.object.configs_filled = True
        self.object.save(update_fields=["configs_filled"])
        return super(ElectionDetailConfigsView, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy("management:election-detail-calendar", kwargs={
            'slug': self.kwargs.get('slug'),
            'slug_election': self.kwargs.get('slug_election')
        })


@method_decorator(login_required, name="dispatch")
class ElectionDetailCalendarView(ElectionManagePermissionMixin, FormJsonView):
    """ """
    model = Election
    form_class = ElectoralCalendarForm
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_calendar.html"

    def form_valid(self, form):
        self.object.calendar_filled = True
        self.object.save(update_fields=["calendar_filled"])
        return super(ElectionDetailCalendarView, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy("management:election-detail-categories", kwargs={
            'slug': self.kwargs.get('slug'),
            'slug_election': self.kwargs.get('slug_election')
        })


@method_decorator(login_required, name="dispatch")
class ElectionDetailCategoriesView(ElectionManagePermissionMixin,
                                   FormsetJsonView):
    """ """
    model = Election
    prefix = "categories"
    form_class = CategoryFormset
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_categories.html"

    def get_success_url(self):
        return reverse_lazy("management:election-detail-rolls", kwargs={
            'slug': self.kwargs.get('slug'),
            'slug_election': self.kwargs.get('slug_election')
        })


@method_decorator(login_required, name="dispatch")
class ElectionDetailRollsView(ElectionManagePermissionMixin, SingleTableMixin,
                              FormJsonView):
    """ """
    model = Election
    action = AjaxResponseAction.REFRESH
    form_class = ElectoralRollDocumentForm
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_rolls.html"

    table_class = ElectoralRollTable
    table_pagination = {'per_page': 15}

    def form_valid(self, form):
        """ """
        document = form.save()
        electoral_roll = ElectoralRoll.objects.create(
            election=self.object, document=document)
        import_electoral_roll.delay(electoral_roll.pk)
        return self.json_to_response()

    def get_form_kwargs(self):
        kwargs = super(ElectionDetailRollsView, self).get_form_kwargs()
        kwargs.update({
            "election": self.object,
            "is_importing": ElectoralRoll.objects.is_importing(self.object.pk)
        })
        return kwargs

    def get_table_data(self):
        return self.object.electoral_rolls.all()


@method_decorator(login_required, name="dispatch")
class ElectoralRollPublishView(PermissionRequiredMixin, SingleObjectMixin,
                               View):
    """ """
    model = ElectoralRoll
    permission_required = 'evoteprocesses.manage_electoralprocess'

    def post(self, request, *args, **kwargs):
        electoral_roll = self.get_object()
        electoral_roll.published_on = timezone.now()
        electoral_roll.save(update_fields=['published_on'])
        return redirect('management:election-detail-rolls',
                        slug=electoral_roll.election.electoral_process.slug,
                        slug_election=electoral_roll.election.slug)

    def get_permission_object(self):
        pobject = super(ElectoralRollPublishView, self).get_permission_object()
        return pobject.election.electoral_process


@method_decorator(login_required, name="dispatch")
class ElectionDetailCandidatesView(ElectionManagePermissionMixin,
                                   FormJsonView):
    """ """
    model = Election
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_candidates.html"

    def get_prefix(self):
        """ """
        if not self.object.write_in:
            return "candidates"
        return "document"

    def get_form_class(self):
        """ """
        if not self.object.write_in:
            return CandidateFormset
        return ElectionWriteInDocumentForm

    def get_form_kwargs(self):
        """ """
        kwargs = super(ElectionDetailCandidatesView, self).get_form_kwargs()
        if self.object.write_in:
            kwargs.update({
                "election": self.object,
                "instance": self.object.write_in_import
            })
        else:
            kwargs.update({
                "queryset":
                    self.object.candidates.not_write_in().order_by('created')
            })
        return kwargs

    def form_valid(self, form):
        """ """
        instance = form.save()
        if self.object.write_in:
            if not self.object.write_in_import:
                self.object.write_in_import = instance
                self.object.save()
            importer = CandidateImporter(self.object)
            importer.run(instance.document)
        return self.json_to_response()

    def form_invalid(self, form, prefix=None):
        """ """
        if not self.object.write_in:
            return self.formset_invalid(form, prefix)
        return super(ElectionDetailCandidatesView, self)\
            .form_invalid(form, prefix)

    def get_success_url(self):
        return reverse_lazy("management:election-detail-bulletin", kwargs={
            'slug': self.kwargs.get('slug'),
            'slug_election': self.kwargs.get('slug_election')
        })


@method_decorator(login_required, name="dispatch")
class ElectionDetailBulletinView(ElectionManagePermissionMixin, FormJsonView):
    """ """
    model = Election
    prefix = "bulletin"
    form_class = ElectionBulletinForm
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_bulletin.html"

    def get_success_url(self):
        return reverse_lazy("management:election-list", kwargs={
            'slug': self.kwargs.get('slug'),
        })


@method_decorator(login_required, name="dispatch")
class ElectionDetailBulletinPreviewView(ElectionVotingStep1View):
    """ """
    template_name = "management/election_detail_bulletin_preview.html"

    def get_context_data(self, **kwargs):
        context = super(ElectionDetailBulletinPreviewView, self)\
            .get_context_data(**kwargs)

        context.update({
            'close_url': reverse_lazy(
                "management:election-detail-bulletin", kwargs={
                    'slug': self.kwargs.get('slug'),
                    'slug_election': self.kwargs.get('slug_election')
                })
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectoralRollDetailView(SingleTableMixin, DetailView):
    """ """
    model = ElectoralRoll
    table_class = ElectionVoterTable
    template_name = "management/electoralroll_detail.html"

    def get_table_data(self):
        """ """
        return self.object.election_voters.all()


@method_decorator(login_required, name="dispatch")
class PersonalDataView(PersonalDataBaseView):
    """ """
    template_name = "management/personal_data.html"


class MessagesRecievedListView(MessageMixin, MessagesRecievedView):
    """ """

    def get_queryset(self):
        slug = self.kwargs.get('slug')
        process = get_object_or_404(ElectoralProcess, slug=slug)
        return Message.objects.inbox(self.request.user, process)


class MessagesSentListView(MessageMixin, MessagesSentView):
    """ """

    def get_queryset(self):
        slug = self.kwargs.get('slug')
        process = get_object_or_404(ElectoralProcess, slug=slug)
        return Message.objects.sentbox(self.request.user, process)


class MessagesArchivedListView(MessageMixin, MessagesArchivedView):
    """ """

    def get_queryset(self):
        slug = self.kwargs.get('slug')
        process = get_object_or_404(ElectoralProcess, slug=slug)
        return Message.objects.archivebox(self.request.user, process)


class MessageNewManageView(MessageViewManageMixin, MessageMixin,
                           MessageNewView):
    """ """

    def get_success_url(self):
        slug = self.kwargs.get('slug')
        return nested_url('messages-recieved', slug=slug)

    def form_valid(self, form):
        slug = self.kwargs.get('slug')
        instance = form.save(commit=False)
        form_recipients = form.cleaned_data.get('recipients', [])
        recipients = []
        for name in form_recipients:
            recipients += recipient_register.get_all_users(name)
        instance.save()
        instance.recipients = recipients
        process = get_object_or_404(ElectoralProcess, slug=slug)
        instance.content_object = process
        instance.save()
        if instance.sender:
            assign_perm("view_message", instance.sender, instance)
        Message.objects.send(instance)
        messages.success(
            self.request, _("Your message was sent successfully."))
        return self.json_to_response()


class MessageManageView(MessageMixin, MessageView):
    """ """


class ArchiveMessageManageView(MessageMixin, ArchiveMessageView):
    """ """


class DeleteMessageManageView(MessageMixin, DeleteMessageView):
    """ """


class MessageResponseManageView(MessageViewManageMixin, MessageMixin,
                                MessageResponseView):
    """ """

    def get_success_url(self):
        slug = self.kwargs.get('slug')
        return nested_url('messages-recieved', slug=slug)

    def form_valid(self, form):
        slug = self.kwargs.get('slug', None)
        instance = form.save(commit=False)
        if 'msg_slug' in self.kwargs:
            parent = self.get_permission_object()
            instance.parent = parent
        instance.save()
        instance.recipients = [parent.sender]
        if slug:
            process = get_object_or_404(ElectoralProcess, slug=slug)
            instance.content_object = process
            instance.save()
        if instance.sender:
            assign_perm("view_message", instance.sender, instance)
        Message.objects.send(instance)
        messages.success(
            self.request, _("Your message was sent successfully."))
        return self.json_to_response()


@method_decorator(login_required, name="dispatch")
class GenerateCertificateView(FormJsonSimpleView):
    """ """
    form_class = CertificateGenerationForm
    form_pres_class = CertificateGenerationPresForm
    template_name = "management/generate_certificate.html"

    def get_success_url(self):
        obj = get_object_or_404(
            ElectoralProcess, slug=self.kwargs.get('slug'))
        if obj.is_draft():
            return reverse_lazy(
                "management:electoralprocess-confirmation", args=[obj.slug])
        return reverse_lazy(
            "management:electoralprocess-members-edit", args=[obj.slug])

    def get_context_data(self, **kwargs):
        self.process = get_object_or_404(
            ElectoralProcess, slug=self.kwargs.get('slug'))
        context = super(GenerateCertificateView, self).get_context_data(
            **kwargs)
        context.update({
            'process': self.process,
            'form_pres': self.form_pres_class(process=self.process)
        })
        return context

    def form_valid(self, form):
        obj = get_object_or_404(
            ElectoralProcess, slug=self.kwargs.get('slug'))
        for member in obj.electoral_members.all():
            MembersKeysNotificationMail({'key': member.private_key}).send(
                [(member.user.get_full_name(), member.user.get_email())])
        messages.success(self.request, _("Operations confirmed."))
        return self.json_to_response()
