import requests
import json
import base64

from django.apps import apps
from django.conf import settings
from rest_framework.renderers import JSONRenderer
from evotevotings.admin.utils import get_certificates


class ASrouter(object):
    """ Helper for Anonymizer Service communication handling """
    # Anonymizer service endpoints
    anon_endpoint = False
    election_shard = "election/"
    initialize_shard = "initialize/"
    reference_shard = "references/"
    partial_count_shard = "count/"

    def __init__(self):
        # Get all endpoints from the settings
        services_endpoints = getattr(settings,
                                     'EVOTE_SERVICES_ENDPOINTS',
                                     False)
        # If defined, try to set the anonymizer service base url
        if(services_endpoints):
            self.anon_endpoint = services_endpoints.get(
                                         'anonymizer',
                                         False)

    def build_election_url(self, election_id):
        """ Method to build base endpoint URL for a
        given election ID """
        url = False
        # Fetch the references from the Anonymizer Service
        if(self.anon_endpoint and self.election_shard):
            url = (self.anon_endpoint + self.election_shard +
                   str(election_id) + "/")

        else:
            raise Exception("No Anonymizer endpoint settings")

        return url

    def initialize_election(self, election_id=None, voters_nr=0):
        """ Method to handle election initialization """
        # Sanity check
        if election_id is None or voters_nr == 0:
            return False

        return_value = False
        base_url = self.build_election_url(election_id)

        if(base_url):
            # Build the endpoint URL
            url = base_url + self.initialize_shard
            # Create the required POST data
            request_data = {'voters_nr': voters_nr}
            # Connect to the endpoint
            response = requests.post(url,
                                     data=request_data,
                                     timeout=None)
            # Parse the response
            json_response = response.json()

            parsed_response = json.loads(json_response)

            # Error found, raise exception
            if(parsed_response['error'] != 0):
                raise Exception(parsed_response['message'])

            # If everything checks out
            return_value = True

        else:
            raise Exception("No initialize election endpoint settings")

        return return_value

    def list_references(self, election_id=None, page=1, search=None):
        """ Method to handle references search and listing """
        # List to hold the returned references
        references = []
        more = False
        # Sanity check
        if election_id is None:
            return (references, more)

        base_url = self.build_election_url(election_id)

        if(base_url):
            # Build the endpoint URL
            url = base_url + self.reference_shard
            # Create the required GET data
            request_data = {'page': page}
            if search is not None:
                request_data['unique_vote_bulletin'] = search

            # Connect to the endpoint
            response = requests.get(url, request_data)
            reference_list = response.json()

            # Check for valid response
            if response.status_code == 200 and reference_list['count']:
                for reference in reference_list['results']:
                    references.append(reference['unique_vote_bulletin'])

                # Check for next page
                if reference_list['next'] is not None:
                    more = True

        else:
            raise Exception("No search references endpoint settings")

        return (references, more)

    def partial_count(self, election_id=None):
        """ Method to get the current vote count """
        # Sanity check
        if election_id is None:
            raise Exception("No election ID provided")

        # Return value
        count = 0

        base_url = self.build_election_url(election_id)

        if(base_url):
            # Build the endpoint URL
            url = base_url + self.partial_count_shard

            # Connect to the endpoint
            response = requests.get(url)

            # Parse the response
            json_response = response.json()

            parsed_response = json.loads(json_response)

            # Check for valid response
            if response.status_code == 200 and parsed_response['count']:
                count = parsed_response['count']

        else:
            raise Exception("No count votes endpoint settings")

        return count


class CSrouter(object):
    """ Helper for Counter Service communication handling """
    # Counter service endpoints
    counter_endpoint = False
    scrutiny_shard = "scrutiny/"
    generate_shard = "generate/"
    status_shard = "status/"
    clean_shard = "clean/"

    def __init__(self):
        # Get all endpoints from the settings
        services_endpoints = getattr(settings,
                                     'EVOTE_SERVICES_ENDPOINTS',
                                     False)
        # If defined, try to set the anonymizer service base url
        if(services_endpoints):
            self.counter_endpoint = services_endpoints.get(
                                         'counter',
                                         False)

    def get_election_model(self):
        return apps.get_model("evoteelections.Election")  # noqa

    def build_scrutiny_url(self, election_id):
        """ Method to build base endpoint URL for all
        election scrutiny operations given an election ID """
        url = False
        # Fetch the necessary settings
        if(self.counter_endpoint and self.scrutiny_shard):
            url = (self.counter_endpoint + self.scrutiny_shard +
                   str(election_id) + "/")

        else:
            raise Exception("No Counter Service endpoint settings")

        return url

    def generate_results(self, election_id=None, elect_process=None):
        """ Method to generate the election results """
        # Sanity check
        if election_id is None or elect_process is None:
            raise Exception("No election ID provided")

        base_url = self.build_scrutiny_url(election_id)

        results = None

        if(base_url):
            # Build the endpoint URL
            url = base_url + self.generate_shard

            # Fetch all data to be sent to the counter service
            secret_entries = get_certificates(election_id)

            # Election private key (from Election)
            elec_pvt_key = secret_entries['election_pvt_key']

            # Counter Service private key
            cs_pvt_key = secret_entries['cs_pvt_key']

            # Election management certificate
            sge_cert = secret_entries['sge']

            # Validator Service certificate
            vs_cert = secret_entries['vs']

            # Root (CA) certificate
            root_cert = secret_entries['root']

            # List of members shared secrets
            secret_list = []

            # Electoral Process ID
            Election = self.get_election_model()
            election = Election.objects.get(pk=election_id)
            elec_proc_id = election.electoral_process.pk

            # Build the request POST data
            # Encoding all certificates and private keys
            request_data = {
                'election_pvt_key': base64.b64encode(elec_pvt_key),
                'cs_pvt_key': base64.b64encode(cs_pvt_key),
                'vs_cert': base64.b64encode(vs_cert),
                'root_cert': base64.b64encode(root_cert),
                'sge_cert': base64.b64encode(sge_cert),
                'secret_list': secret_list,
                'elec_proc_id': elec_proc_id
            }
            response = requests.post(
                url,
                JSONRenderer().render(request_data),
                headers={'content-type': 'application/json'}
            )

            json_response = json.loads(response.json())

            if response.status_code == 200:
                # Check for errors
                if json_response['error']:
                    raise Exception(json_response['message'])
                else:
                    if json_response['message'] is not False:
                        results = json_response['message']
            else:
                results = json_response

        else:
            raise Exception("No check results votes endpoint settings")

        return results

    def check_results_status(self, election_id=None):
        """ Method to check the current scrutiny status """
        # Sanity check
        if election_id is None:
            raise Exception("No election ID provided")

        # Return value
        status = 0

        base_url = self.build_election_url(election_id)

        if(base_url):
            # Build the endpoint URL
            url = base_url + self.status_shard

            # Connect to the endpoint
            response = requests.get(url)

            # Parse the response
            json_response = response.json()

            parsed_response = json.loads(json_response)

            # Check for valid response
            if (response.status_code == 200
               and parsed_response['result_status_choices']):
                status = int(parsed_response['result_status_choices'])

        else:
            raise Exception("No check results votes endpoint settings")

        return status

    def clean_results(self, election_id=None):
        """ Method to clean all results associated to a
            given election ID.
            Returns the response status code """
        # Sanity check
        if election_id is None:
            raise Exception("No election ID provided")

        base_url = self.build_election_url(election_id)

        # Defaults to 400 bad request
        response_code = 400

        if(base_url):
            # Build the endpoint URL
            url = base_url + self.clean_shard

            # Connect to the endpoint
            response = requests.get(url)

            # Set the response code to return
            response_code = response.status_code

        else:
            raise Exception("No check results votes endpoint settings")

        return response_code
