from __future__ import unicode_literals

import json
from unittest import skip

from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.test import SimpleTestCase, TestCase
from django.conf import settings
from mock import patch

from evotevotings.elections.tests.utils import ElectionsTestUtils
from evoteusers.tests.utils import UserTestUtils

from . import VotersTestCase


class LoginViewTest(VotersTestCase):
    """ """

    def setUp(self):
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.user = self.create_user(**self.credentials)
        self.url = reverse('voters:login')

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/login.html')

    def test_login_valid_user(self):
        """ """
        response = self.client.post(self.url, follow=True,
                                    data=self.credentials)
        self.assertRedirects(response, settings.LOGIN_REDIRECT_URL)
        self.assertEqual(response.context[0]['user'], self.user)

    def test_login_with_wrong_credentials(self):
        self.credentials['password'] = "wrong_password"
        response = self.client.post(self.url, data=self.credentials)
        error_message = "Please enter a correct email and password. " +\
            "Note that both fields may be case-sensitive."
        self.assertFormError(response, "form", None, error_message)


class LogoutViewTest(VotersTestCase):

    def setUp(self):
        self.login()

    def test_logout_view(self):
        """ """
        response = self.client.get(reverse('voters:logout'), follow=True)
        self.assertRedirects(response, settings.LOGIN_URL)
        self.assertIsInstance(response.context[0]['user'], AnonymousUser)


class ElectoralProcessListViewTest(VotersTestCase):

    def setUp(self):
        # create and login user
        self.login()
        self.url = reverse("voters:electoralprocess-list")

    def test_used_template_to_unauthenticated(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "voters/electoralprocess_list.html")

    def test_used_template_to_authenticated(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/electoralprocess_mylist.html")

    @skip("to implement")
    def test_context_table_class_to_unauthenticated(self):
        pass

    @skip("to implement")
    def test_context_table_class_to_authenticated(self):
        pass

    @skip("to implement")
    def test_context_queryset_to_unauthenticated(self):
        pass

    @skip("to implement")
    def test_context_queryset_to_authenticated(self):
        pass


class ElectoralProcessDetailViewTest(VotersTestCase):

    def setUp(self):
        self.login_as_voter()
        self.url = reverse("voters:electoralprocess-detail", kwargs={
            "slug": self.voter.election.electoral_process.slug
        })

    def test_used_template_to_authenticated(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/electoralprocess_mydetail.html")

    def test_used_template_to_unauthenticated(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/electoralprocess_detail.html")


class ElectoralProcessArchiveViewTest(VotersTestCase):

    def test_used_template(self):
        """ """
        response = self.client.get(reverse("voters:electoralprocess-archive"))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/electoralprocess_archive.html")

    @skip("to implement")
    def test_context_table_class(self):
        """TODO: implement test"""
        pass

    @skip("to implement")
    def test_context_queryset(self):
        """TODO: implement test"""
        pass


class ElectionHistoryViewTest(VotersTestCase):

    def setUp(self):
        self.login()
        self.url = reverse("voters:election-history")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "voters/election_history.html")

    def test_get_to_unauthenticated_user(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    @skip("to implement")
    def test_context_table_class(self):
        """TODO: implement test"""
        pass

    @skip("to implement")
    def test_context_queryset(self):
        """TODO: implement test"""
        pass


class ElectoralProcessVotingViewTest(VotersTestCase):

    def setUp(self):
        self.login_as_voter()
        self.url = reverse("voters:electoralprocess-voting", kwargs={
            "slug": self.voter.election.electoral_process.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, 'voters/electoralprocess_voting.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionVotingStep1ViewTest(VotersTestCase):

    def setUp(self):
        self.login_as_voter()
        self.url = reverse("voters:election-voting-step1", kwargs={
            "slug": self.voter.election.electoral_process.slug,
            "slug_election": self.voter.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_voting_step1.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_invalid_context_data(self):
        """ """
        response = self.client.get(self.url)
        context = response.context[-1]
        self.assertIn("is_valid", context)
        self.assertFalse(context['is_valid'])

    @patch("evotevotings.voters.views.get_extra_election_data")
    def test_context_data(self, mock_fn):
        """ """
        mock_fn.return_value = {
            'combo_id': 'dummy',
            'vs_cert': 'dummy',
            'election_id': 'dummy'
        }

        response = self.client.get(self.url)
        context = response.context[-1]
        self.assertIn("is_valid", context)
        self.assertTrue(context['is_valid'])


class ElectionVotingStep2ViewTest(VotersTestCase):

    def setUp(self):
        self.login_as_voter()
        self.url = reverse("voters:election-voting-step2", kwargs={
            "slug": self.voter.election.electoral_process.slug,
            "slug_election": self.voter.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_voting_step2.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionVotingStep3ViewTest(VotersTestCase):

    def setUp(self):
        self.login_as_voter()
        self.url = reverse("voters:election-voting-step3", kwargs={
            "slug": self.voter.election.electoral_process.slug,
            "slug_election": self.voter.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_voting_step3.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionVotingStep3PinViewTest(VotersTestCase):

    def setUp(self):
        self.login_as_voter()
        self.url = reverse("voters:election-voting-step3-pin", kwargs={
            "slug": self.voter.election.electoral_process.slug,
            "slug_election": self.voter.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response,
                                'voters/election_voting_step3_pin.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionVotingStep4ViewTest(VotersTestCase):

    def setUp(self):
        self.login_as_voter()
        self.url = reverse("voters:election-voting-step4", kwargs={
            "slug": self.voter.election.electoral_process.slug,
            "slug_election": self.voter.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_voting_step4.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionSearchVoteViewTest(VotersTestCase):

    def setUp(self):
        self.login_as_voter()
        self.election = self.voter.election
        self.references = [
            'REF#1',
            'UREF#2',
            'TREF#3',
            'REF#4',
        ]
        kwargs = {
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        }
        self.url = reverse("voters:election-search-vote",
                           kwargs=kwargs)

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_search_vote.html')

    @patch("evotevotings.voters.views.get_references")
    def test_reference_data(self, mock_fn):
        """ """
        mock_fn.return_value = (self.references, False)
        response = self.client.get(self.url)
        context = response.context[-1]
        self.assertIn("references", context)
        self.assertEqual(context['references'], self.references)

    @patch("evotevotings.voters.views.get_references")
    def test_reference_search(self, mock_fn):
        """ """
        mock_fn.return_value = ([self.references[1], ], False)
        search_data = {'search': self.references[1]}
        response = self.client.get(self.url, search_data)
        context = response.context[-1]
        self.assertIn("references", context)
        self.assertEqual(context['references'], [self.references[1], ])

    @patch("evotevotings.voters.views.get_references")
    def test_reference_pagination(self, mock_fn):
        """ """
        mock_fn.return_value = (self.references, True)
        response = self.client.get(self.url)
        context = response.context[-1]
        self.assertIn("references", context)
        self.assertIn("more", context)
        self.assertTrue(context['more'])
        self.assertIn("next_page", context)


class ContactsViewTest(SimpleTestCase):

    def setUp(self):
        self.url = reverse("voters:contacts")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/contacts.html')


def raise_valerror(self, *args, **kwargs):
    self.add_error('notice_email', ('Please provide the notice email'))


class HowitWorksViewTest(SimpleTestCase):
    def setUp(self):
        self.url = reverse("voters:howitworks")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/howitworks.html')


class PersonalDataViewTest(VotersTestCase):
    def setUp(self):
        self.login()
        self.url = reverse("voters:personal-data")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/personal_data.html')

    def test_form_submit_valid(self):
        data = {
            'personal-photo': 'Photo',
            'personal-name': 'Name',
            'personal-email': 'test@test.com',
            'personal-primary_phone': '+351911111111',
            'personal-notice_method': 1,
            'personal-notice_phone': '',
            'personal-notice_email': 'test@test.com',
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    @patch("evoteusers.forms.PersonalDataForm.clean", new=raise_valerror)
    def test_form_submit_fail_mail(self):
        data = {
            'personal-photo': 'Photo',
            'personal-name': 'Name',
            'personal-primary_phone': '+351911111111',
            'personal-notice_method': 1,
            'personal-notice_phone': '',
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('errors_list', content)

    def test_form_submit_fail_phone(self):
        data = {
            'personal-photo': 'Photo',
            'personal-name': 'Name',
            'personal-email': 'test@test.com',
            'personal-primary_phone': '',
            'personal-notice_method': 0,
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('errors_list', content)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionClaimViewTest(UserTestUtils, ElectionsTestUtils, TestCase):

    def setUp(self):
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.credentials_reciever = {
            "email": "dev_reciever@eurotux.com",
            "password": "password"
        }
        self.user = self.create_user(**self.credentials)
        self.reciever = self.create_user(**self.credentials_reciever)
        self.election = self.create_election()
        self.url = reverse("voters:claim", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        """ """
        self.client.login(**self.credentials)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, 'voters/evotemessages/message_form.html')

    def test_form_submit_valid_as_authenticated(self):
        data = {
            'claimform-name': self.user.name,
            'claimform-email': self.user.email,
            'claimform-recipients': [self.reciever.pk],
            'claimform-subject':
                'Claim - Election : ' + self.election.identifier,
            'claimform-body': 'body of message',
        }
        self.client.login(**self.credentials)
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    def test_form_submit_valid_as_anonymous(self):
        data = {
            'claimform-name': 'Anonymous user',
            'claimform-anonymous_email': 'anonymous@server.com',
            'claimform-recipients': [self.reciever.pk],
            'claimform-subject':
                'Claim - Election : ' + self.election.identifier,
            'claimform-body': 'body of message',
            'g-recaptcha-response': 'PASSED',
            'recaptcha_response_field': 'PASSED'
        }
        self.client.logout()
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)
