from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse_lazy
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.utils.decorators import method_decorator
from django.views.generic import ListView, DetailView, TemplateView
from django.contrib import messages
from django.conf import settings
from django.shortcuts import get_object_or_404

from guardian.shortcuts import assign_perm


from evotevotings.processes.models import ElectoralProcess
from evotevotings.elections.models import Election
from evotevotings.elections.utils import get_extra_election_data
from evoteajaxforms.views import FormJsonSimpleView, AjaxResponseAction
from evotemessages.mixins import MessageBaseMixin, MessageFormBaseMixin
from evotemessages.models import Message
from evoteusers.views import EvoteLoginView, PersonalDataBaseView
from evotecore.mixins import (DjangoTablesFilterMixin, PermissionListMixin,
                              PermissionRequiredMixin)
from evotecore.decorators import search_register

from .tables import (ElectoralProcessTable, ElectoralProcessArchiveTable,
                     ElectoralProcessMyTable, ElectionHistoryTable)
from .filters import ArchiveFilter, ElectionHistoryFilter, IndexFilter
from .forms import (ElectionVotingStep1Form, ElectionSearchVoteForm,
                    ContactsForm)
from evotevotings.voters.utils import get_references
from .mailings import ContactMail
from .middleware import SESSION_USER_PROFILE


class LoginView(EvoteLoginView):
    """Provides user the ability to login"""
    template_name = "voters/login.html"

    def form_valid(self, form):
        self.request.session[SESSION_USER_PROFILE] = \
            form.cleaned_data.get('profile')
        return super(LoginView, self).form_valid(form)


class ElectoralProcessListView(
        PermissionListMixin, DjangoTablesFilterMixin, ListView):
    """
    Shows the main view
    """
    model = ElectoralProcess
    queryset = ElectoralProcess.objects.published()
    permission_required = "evoteprocesses.view_electoralprocess"
    # tables
    table_pagination = {'per_page': 5}
    # filters
    filter_class = IndexFilter

    def get_table_class(self):
        if self.request.user.is_authenticated():
            return ElectoralProcessMyTable
        return ElectoralProcessTable

    def get_template_names(self):
        if self.request.user.is_authenticated():
            return ["voters/electoralprocess_mylist.html"]
        return ["voters/electoralprocess_list.html"]


class ElectoralProcessDetailView(PermissionRequiredMixin, DetailView):
    """
    Shows the Electoral Process Detail view
    """
    model = ElectoralProcess
    permission_required = "evoteprocesses.view_electoralprocess"

    def get_template_names(self):
        if self.request.user.is_authenticated():
            return ["voters/electoralprocess_mydetail.html"]
        return ["voters/electoralprocess_detail.html"]

    def get_context_data(self, **kwargs):
        context = super(ElectoralProcessDetailView, self)\
            .get_context_data(**kwargs)
        context.update({
            'show_vote_link': self.object.elections.eligible(self.request.user)
            .in_to_vote().exists()
        })
        return context


class ElectoralProcessArchiveView(
        PermissionListMixin, DjangoTablesFilterMixin, ListView):
    """
    Shows the archive view
    """
    model = ElectoralProcess
    queryset = ElectoralProcess.objects.closed()
    permission_required = "evoteprocesses.view_electoralprocess"
    template_name = "voters/electoralprocess_archive.html"
    # tables
    table_class = ElectoralProcessArchiveTable
    table_pagination = {'per_page': 5}
    # filters
    filter_class = ArchiveFilter


@method_decorator(login_required, name="dispatch")
class ElectionHistoryView(
        PermissionListMixin, DjangoTablesFilterMixin, ListView):
    """
    Shows the my elections history view
    """
    model = Election
    queryset = Election.objects.closed()
    permission_required = "evoteelections.view_election"
    template_name = "voters/election_history.html"
    # tables
    table_class = ElectionHistoryTable
    table_pagination = {'per_page': 5}
    # filters
    filter_class = ElectionHistoryFilter


@method_decorator(login_required, name="dispatch")
class ElectoralProcessVotingView(PermissionRequiredMixin, DetailView):
    """
    Shows the voting system
    """
    model = ElectoralProcess
    permission_required = "evoteprocesses.view_electoralprocess"
    template_name = "voters/electoralprocess_voting.html"

    def get_context_data(self, **kwargs):
        """ """
        context = super(ElectoralProcessVotingView, self)\
            .get_context_data(**kwargs)
        elections = self.object.elections.eligible(self.request.user)\
            .in_to_vote()
        context.update({
            'elections': elections,
            'elections_active_count': elections.count(),
            'close_url': reverse_lazy(
                "voters:electoralprocess-detail", kwargs={
                    'slug': self.object.slug
                })
        })
        return context


class ElectionVotingStep(PermissionRequiredMixin, DetailView):
    """
    Generic class for all steps
    """
    model = Election
    permission_required = "evoteelections.vote_election"
    slug_url_kwarg = "slug_election"

    def get_context_data(self, **kwargs):
        context = super(ElectionVotingStep, self).get_context_data(**kwargs)
        context.update({
            'close_url': reverse_lazy(
                "voters:electoralprocess-detail", kwargs={
                    'slug': self.object.electoral_process.slug
                })
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep1View(ElectionVotingStep):
    """
    Shows the voting system (step 1)
    """
    form_class = ElectionVotingStep1Form
    template_name = "voters/election_voting_step1.html"

    def get_context_data(self, **kwargs):
        context = super(ElectionVotingStep1View, self)\
            .get_context_data(**kwargs)

        form_kwargs = {'instance': self.object}
        self.elect_data = get_extra_election_data(self.object.pk,
                                                  self.request.user.pk)
        if self.elect_data is not False:
            form_kwargs.update(self.elect_data)

        context.update({
            'form': self.form_class(**form_kwargs),
            'is_valid': bool(self.elect_data)
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep2View(ElectionVotingStep):
    """
    Shows the voting system (step 2)
    """
    template_name = "voters/election_voting_step2.html"


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep3View(ElectionVotingStep):
    """
    Shows the voting system (step 3)
    """
    template_name = "voters/election_voting_step3.html"


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep3PinView(ElectionVotingStep):
    """
    Shows the voting system (step 3)
    """
    template_name = "voters/election_voting_step3_pin.html"


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep4View(ElectionVotingStep):
    """
    Shows the voting system (step 4)
    """
    template_name = "voters/election_voting_step4.html"


@method_decorator(login_required, name="dispatch")
class ElectionSearchVoteView(PermissionRequiredMixin, DetailView):
    """
    Seach vote in some election
    """
    model = Election
    form_class = ElectionSearchVoteForm
    permission_required = "evoteelections.vote_election"
    template_name = "voters/election_search_vote.html"
    slug_url_kwarg = "slug_election"

    def get_context_data(self, **kwargs):
        """ """
        context = super(ElectionSearchVoteView, self).\
            get_context_data(**kwargs)

        page = self.request.GET.get('page', 1)
        election_id = self.object.pk
        electoral_process_id = self.object.electoral_process_id
        search_text = self.request.GET.get('search') or ''

        (references, more) = get_references(
            election_id,
            page,
            search_text)

        context.update({
            'form': self.form_class(self.request.GET),
            'references': references,
            'search': search_text,
            'election_id': election_id,
            'electoral_process_id': electoral_process_id,
            'page': page,
            'next_page': int(page) + 1,
            'more': more
        })

        return context


class ContactsView(FormJsonSimpleView):
    """ """
    prefix = 'contacts'
    form_class = ContactsForm
    template_name = "voters/contacts.html"
    action = AjaxResponseAction.REFRESH

    def form_valid(self, form):
        context = {
            'name': form.cleaned_data.get('name', ''),
            'from': form.cleaned_data.get('email', ''),
            'subject': form.cleaned_data.get('subject', ''),
            'message': form.cleaned_data.get('message', '')
        }
        ContactMail(context).send()
        messages.success(
            self.request, _("Your request was submitted successfully."))
        return super(ContactsView, self).form_valid(form)

    def get_context_data(self, **kwargs):
        """ """
        context = super(ContactsView, self).get_context_data(**kwargs)

        context['contact_name'] = getattr(
            settings, "EVOTE_CONTACT_NAME", None)
        email = getattr(
            settings, "EVOTE_CONTACT_MAIL", None)
        if email:
            context['contact_mail'] = email[0][1]
        context['contact_phone'] = getattr(
            settings, "EVOTE_CONTACT_PHONE", None)
        context['contact_address'] = getattr(
            settings, "EVOTE_CONTACT_ADDRESS", None)
        context['contact_map_address'] = getattr(
            settings, "EVOTE_CONTACT_ADDRESS", False)
        context['contact_map_gps'] = getattr(
            settings, "EVOTE_CONTACT_GPS",
            {'lat': 41.5598965, 'lng': -8.3988118})
        return context


@method_decorator(login_required, name="dispatch")
class PersonalDataView(PersonalDataBaseView):
    """ """
    template_name = "voters/personal_data.html"


class SearchView(TemplateView):
    """ View showing search results from the base search query """
    template_name = "voters/search.html"

    def get_context_data(self, **kwargs):
        """  """
        context = super(SearchView, self).get_context_data(**kwargs)
        # Search the models based on the GET 'q' parameter value
        # as query string
        query_string = self.request.GET.get('q', "")

        # Only search if a valid query string exists.
        if query_string != "":

            # Split each word of the querystring into a list
            terms = query_string.split()

            results = []

            # Iterate over the search register in order to detect all the
            # classes that have the "searchable" annotation.
            for r in search_register:
                results += r.search(terms)

            # Prepare pagination
            page = self.request.GET.get('page')
            # Define pagination (and number of results p/ page)
            paginator = Paginator(results, 10)
            try:
                results = paginator.page(page)
            except PageNotAnInteger:
                # If not a valid page param, return the first page.
                results = paginator.page(1)
            except EmptyPage:
                # If page out of range, return last
                results = paginator.page(paginator.num_pages)

            context['query'] = query_string
            context['terms'] = terms
            context['results'] = results

        return context


class ElectionClaimView(MessageBaseMixin, MessageFormBaseMixin,
                        FormJsonSimpleView):
    """ """
    prefix = 'claimform'
    template = "evotemessages/message_form.html"
    action = AjaxResponseAction.REDIRECT
    claim = True

    def get_success_url(self):
        return reverse_lazy(
            "voters:electoralprocess-detail", args=[self.kwargs['slug']])

    def form_valid(self, form):
        slug = self.kwargs.get('slug', None)
        instance = form.save()
        if slug:
            process = get_object_or_404(ElectoralProcess, slug=slug)
            instance.content_object = process
            instance.save()
            instance.recipients.add(
                *[member.user for member in process.electoral_members.all()])
            if instance.sender:
                assign_perm("view_message", instance.sender, instance)
        name = form.cleaned_data.get('name')
        Message.objects.notify_anonymous_sender(instance, name)
        Message.objects.send(instance)
        messages.success(
            self.request, _("Your claim was sent."))
        return self.json_to_response()

    def get_initial(self):
        initial = super(ElectionClaimView, self).get_initial()
        election = get_object_or_404(
            Election, slug=self.kwargs['slug_election'])
        initial.update({
            'subject': _('Claim - Election ') + election.identifier
        })
        if self.request.user.is_authenticated():
            initial.update({
                'name': self.request.user.name,
                'email': self.request.user.get_email()
            })
        return initial


class HowitworksView(TemplateView):
    """ """
    template_name = "voters/howitworks.html"
